@@ -1,8 +1,23 @@
-#### Triangular Packed Matrices -- Coercion and Methods
+## METHODS FOR CLASS: dtpMatrix
+## dense (packed) triangular matrices with 'x' slot of type "double"
+## ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
+## MJ: no longer needed ... replacement in ./denseMatrix.R
+if(FALSE) {
 setAs("dtpMatrix", "dtrMatrix",
       dtp2dtr <- function(from) .Call(dtpMatrix_as_dtrMatrix, from))
+setAs("matrix", "dtpMatrix",
+      function(from) as(as(from, "dtrMatrix"), "dtpMatrix"))
+setAs("dtpMatrix", "matrix",
+      function(from) as(dtp2dtr(from), "matrix"))
+setAs("pCholesky", "lMatrix",
+      function(from) as(as(from, "dtpMatrix"), "lMatrix"))
+setAs("pBunchKaufman", "lMatrix",
+      function(from) as(as(from, "dtpMatrix"), "lMatrix"))
+} ## MJ
 
+## MJ: no longer needed ... replacement in ./sparseMatrix.R
+if(FALSE) {
 ## Is this needed?  already have coercion to "TsparseMatrix" {FIXME}
 setAs("dtpMatrix", "dtTMatrix",
       function(from) {
@@ -16,24 +31,12 @@ setAs("dtpMatrix", "dtTMatrix",
 		    toClass = "dtTMatrix", do.n = FALSE)
 	  }
       })
+} ## MJ
 
-setAs("dtpMatrix", "matrix",
-      function(from) as(dtp2dtr(from), "matrix"))
-setAs("matrix", "dtpMatrix",
-      function(from) as(as(from, "dtrMatrix"), "dtpMatrix"))
-
-setAs("pCholesky", "lMatrix",
-      function(from) as(as(from, "dtpMatrix"), "lMatrix"))
-setAs("pBunchKaufman", "lMatrix",
-      function(from) as(as(from, "dtpMatrix"), "lMatrix"))
-
-
-setMethod("determinant", signature(x = "dtpMatrix", logarithm = "missing"),
-	  function(x, logarithm, ...) determinant(x, TRUE))
-
-setMethod("determinant", signature(x = "dtpMatrix", logarithm = "logical"),
-	  function(x, logarithm, ...) mkDet(diag(x), logarithm))
-
+## MJ: no longer needed ... replacement in ./packedMatrix.R
+if (FALSE) {
+setMethod("t", "dtpMatrix",
+	  function(x) dtr2dtp(t(dtp2dtr(x))), valueClass = "dtpMatrix")
 setMethod("diag", signature(x = "dtpMatrix"),
 	  function(x, nrow, ncol) .Call(dtpMatrix_getDiag, x),
 	  valueClass = "numeric")
@@ -43,38 +46,4 @@ setMethod("diag<-", signature(x = "dtpMatrix"),
 		    if(x@diag == "U") .dense.diagU2N(x, "d", isPacked=TRUE) else x,
 		    value)
 	  })
-
-setMethod("norm", signature(x = "dtpMatrix", type = "character"),
-	  function(x, type, ...)
-	      if(identical("2", type)) norm2(x) else .Call(dtpMatrix_norm, x, type),
-	  valueClass = "numeric")
-
-setMethod("norm", signature(x = "dtpMatrix", type = "missing"),
-	  function(x, type, ...) .Call(dtpMatrix_norm, x, "O"),
-	  valueClass = "numeric")
-
-setMethod("rcond", signature(x = "dtpMatrix", norm = "character"),
-	  function(x, norm, ...)
-	  .Call(dtpMatrix_rcond, x, norm),
-	  valueClass = "numeric")
-
-setMethod("rcond", signature(x = "dtpMatrix", norm = "missing"),
-	  function(x, norm, ...)
-	  .Call(dtpMatrix_rcond, x, "O"),
-	  valueClass = "numeric")
-
-setMethod("solve", signature(a = "dtpMatrix", b="missing"),
-	  function(a, b, ...) .Call(dtpMatrix_solve, a),
-	  valueClass = "dtpMatrix")
-
-setMethod("solve", signature(a = "dtpMatrix", b="ddenseMatrix"),
-	  function(a, b, ...) .Call(dtpMatrix_matrix_solve, a, b),
-	  valueClass = "dgeMatrix")
-
-setMethod("solve", signature(a = "dtpMatrix", b="matrix"),
-	  function(a, b, ...) .Call(dtpMatrix_matrix_solve, a, b),
-	  valueClass = "dgeMatrix")
-
-## FIXME: speed up
-setMethod("t", "dtpMatrix",
-	  function(x) dtr2dtp(t(dtp2dtr(x))), valueClass = "dtpMatrix")
+} ## MJ
