@@ -2,16 +2,8 @@
 
 ### contains = "lsparseMatrix"
 
-## Can use CsparseMatrix methods for all of these
-
-## setMethod("t", signature(x = "lgCMatrix"),
-##           function(x) .Call(lgCMatrix_trans, x),
-##           valueClass = "lgCMatrix")
-
-## setMethod("diag", signature(x = "lgCMatrix"),
-## 	  function(x, nrow, ncol) .Call(lgCMatrix_diag, x))
-
-
+## MJ: no longer needed ... replacement in ./sparseMatrix.R
+if(FALSE) {
 setAs("lgCMatrix", "dgCMatrix",
       function(from) new("dgCMatrix", i = from@i, p = from@p,
                          x = as.double(from@x),
@@ -29,6 +21,9 @@ setAs("lgCMatrix", "lgeMatrix",
 
 setAs("lgCMatrix", "matrix", function(from) .Call(lgC_to_matrix, from))
 ## not this: .Call(Csparse_to_matrix, from)), since it goes via dense -> double precision
+} ## MJ
 
+## MJ: no longer needed ... replacement in ./denseMatrix.R
+if(FALSE) {
 setAs("matrix", "lgCMatrix", .m2lgC)
-
+} ## MJ
