@@ -6,6 +6,7 @@
 #include "Csparse.h"
 #include "Tsparse.h"
 #include "chm_common.h"
+#include "cs_utils.h" /* -> ./cs.h  for cs_dmperm() */
 
 /** "Cheap" C version of  Csparse_validate() - *not* sorting : */
 Rboolean isValid_Csparse(SEXP x)
@@ -104,6 +105,9 @@ SEXP Rsparse_validate(SEXP x)
     return ScalarLogical(1);
 }
 
+/* MJ: no longer needed ... prefer R_sparse_as_dense() */
+#if 0
+
 /** @brief From a CsparseMatrix, produce a dense one.
  *
  * Directly deals with symmetric, triangular and general.
@@ -135,45 +139,56 @@ SEXP Csparse_to_dense(SEXP x, SEXP symm_or_tri)
 	if(is_sym || is_tri)
 	    ctype = R_check_class_etc(x, valid);
     }
-    CHM_SP chxs = AS_CHM_SP__(x);// -> chxs->stype = +- 1 <==> symmetric
+    CHM_SP chxs = AS_CHM_SP__(x);// -> chxs->stype = +- 1 <==> symmetric // allocated with alloca()
     R_CheckStack();
-    if(is_tri && *diag_P(x) == 'U') { // ==>  x := diagU2N(x), directly for chxs
+    Rboolean is_U_tri = is_tri && *diag_P(x) == 'U';
+    if(is_U_tri) { // ==>  x := diagU2N(x), directly for chxs;  further: must free chxs
 	CHM_SP eye = cholmod_speye(chxs->nrow, chxs->ncol, chxs->xtype, &c);
 	double one[] = {1, 0};
 	CHM_SP ans = cholmod_add(chxs, eye, one, one,
 				 /* values: */ ((ctype / 3) != 2), // TRUE iff not "nMatrix"
 				 TRUE, &c);
 	cholmod_free_sparse(&eye, &c);
-	chxs = cholmod_copy_sparse(ans, &c);
+	chxs = cholmod_copy_sparse(ans, &c); // replacing alloca'd chxs with malloc'ed one, which must be freed
 	cholmod_free_sparse(&ans, &c);
     }
     /* The following loses the symmetry property, since cholmod_dense has none,
      * BUT, much worse (FIXME!), it also transforms CHOLMOD_PATTERN ("n") matrices
      * to numeric (CHOLMOD_REAL) ones {and we "revert" via chm_dense_to_SEXP()}: */
     CHM_DN chxd = cholmod_sparse_to_dense(chxs, &c);
-    int Rkind = (chxs->xtype == CHOLMOD_PATTERN)? -1 : Real_kind(x);
+    /* FIXME: The above FAILS for prod(dim(.)) > INT_MAX
+     * ----
+     * TODO: use cholmod_l_* but also the 'cl' global ==> many changes in chm_common.[ch]
+     * >>>>>>>>>>> TODO <<<<<<<<<<<<
+     * CHM_DN chxd = cholmod_l_sparse_to_dense(chxs, &cl); */
+    //                   ^^^ important when prod(dim(.)) > INT_MAX
+    int chxs_xtype = chxs->xtype;
+    int chxs_stype = chxs->stype;
+    if(is_U_tri)
+       cholmod_free_sparse(&chxs, &c);
+    int Rkind = (chxs_xtype == CHOLMOD_PATTERN)? -1 : Real_kind(x);
 
     SEXP ans = chm_dense_to_SEXP(chxd, 1, Rkind, GET_SLOT(x, Matrix_DimNamesSym),
 				 /* transp: */ FALSE);
     // -> a [dln]geMatrix
     if(is_sym) { // ==> want  [dln]syMatrix
-	const char cl1 = class_P(ans)[0];
 	PROTECT(ans);
-	SEXP aa = PROTECT(NEW_OBJECT(MAKE_CLASS((cl1 == 'd') ? "dsyMatrix" :
-						((cl1 == 'l') ? "lsyMatrix" : "nsyMatrix"))));
+	const char cl1 = class_P(ans)[0];
+	SEXP aa = PROTECT(NEW_OBJECT_OF_CLASS((cl1 == 'd') ? "dsyMatrix" :
+					      ((cl1 == 'l') ? "lsyMatrix" : "nsyMatrix")));
 	// No need to duplicate() as slots of ans are freshly allocated and ans will not be used
 	SET_SLOT(aa, Matrix_xSym,       GET_SLOT(ans, Matrix_xSym));
 	SET_SLOT(aa, Matrix_DimSym,     GET_SLOT(ans, Matrix_DimSym));
 	SET_SLOT(aa, Matrix_DimNamesSym,GET_SLOT(ans, Matrix_DimNamesSym));
-	SET_SLOT(aa, Matrix_uploSym, mkString((chxs->stype > 0) ? "U" : "L"));
+	SET_SLOT(aa, Matrix_uploSym, mkString((chxs_stype > 0) ? "U" : "L"));
 	UNPROTECT(2);
 	return aa;
     }
     else if(is_tri) { // ==> want  [dln]trMatrix
-	const char cl1 = class_P(ans)[0];
 	PROTECT(ans);
-	SEXP aa = PROTECT(NEW_OBJECT(MAKE_CLASS((cl1 == 'd') ? "dtrMatrix" :
-						((cl1 == 'l') ? "ltrMatrix" : "ntrMatrix"))));
+	const char cl1 = class_P(ans)[0];
+	SEXP aa = PROTECT(NEW_OBJECT_OF_CLASS((cl1 == 'd') ? "dtrMatrix" :
+					      ((cl1 == 'l') ? "ltrMatrix" : "ntrMatrix")));
 	// No need to duplicate() as slots of ans are freshly allocated and ans will not be used
 	SET_SLOT(aa, Matrix_xSym,       GET_SLOT(ans, Matrix_xSym));
 	SET_SLOT(aa, Matrix_DimSym,     GET_SLOT(ans, Matrix_DimSym));
@@ -188,6 +203,8 @@ SEXP Csparse_to_dense(SEXP x, SEXP symm_or_tri)
 	return ans;
 }
 
+#endif /* MJ */
+
 // FIXME: do not go via CHM (should not be too hard, to just *drop* the x-slot, right?
 SEXP Csparse2nz(SEXP x, Rboolean tri)
 {
@@ -201,6 +218,10 @@ SEXP Csparse2nz(SEXP x, Rboolean tri)
 			      /* diag = */ tri ? diag_P(x) : "",
 			      GET_SLOT(x, Matrix_DimNamesSym));
 }
+
+/* MJ: no longer needed ... prefer R_sparse_as_kind() */
+#if 0
+
 SEXP Csparse_to_nz_pattern(SEXP x, SEXP tri)
 {
     int tr_ = asLogical(tri);
@@ -217,6 +238,8 @@ SEXP nz_pattern_to_Csparse(SEXP x, SEXP res_kind)
     return nz2Csparse(x, asInteger(res_kind));
 }
 
+#endif /* MJ */
+
 // n.CMatrix --> [dli].CMatrix  (not going through CHM!)
 // NOTE: use chm_MOD_xtype(() to change type of  'cholmod_sparse' matrix
 SEXP nz2Csparse(SEXP x, enum x_slot_kind r_kind)
@@ -240,7 +263,7 @@ SEXP nz2Csparse(SEXP x, enum x_slot_kind r_kind)
     ncl[0] = (r_kind == x_double ? 'd' :
 	      (r_kind == x_logical ? 'l' :
 	       /* else (for now):  r_kind == x_integer : */ 'i'));
-    PROTECT(ans = NEW_OBJECT(MAKE_CLASS(ncl)));
+    PROTECT(ans = NEW_OBJECT_OF_CLASS(ncl));
     // create a correct 'x' slot:
     switch(r_kind) {
 	int i;
@@ -275,6 +298,9 @@ SEXP nz2Csparse(SEXP x, enum x_slot_kind r_kind)
     return ans;
 }
 
+/* MJ: no longer needed ... prefer R_sparse_as_matrix() */
+#if 0
+
 SEXP Csparse_to_matrix(SEXP x, SEXP chk, SEXP symm)
 {
     int is_sym = asLogical(symm);
@@ -287,15 +313,22 @@ SEXP Csparse_to_matrix(SEXP x, SEXP chk, SEXP symm)
 	cholmod_sparse_to_dense(AS_CHM_SP2(x, asLogical(chk)), &c),
 	1 /*do_free*/,
 	(is_sym
-	 ? symmetric_DimNames(GET_SLOT(x, Matrix_DimNamesSym))
-	 :                    GET_SLOT(x, Matrix_DimNamesSym)));
+	 ? get_symmetrized_DimNames(x, -1)
+	 : GET_SLOT(x, Matrix_DimNamesSym)));
 }
 
+#endif /* MJ */
+
+/* MJ: no longer needed ... prefer R_sparse_as_vector() */
+#if 0
+
 SEXP Csparse_to_vector(SEXP x)
 {
     return chm_dense_to_vector(cholmod_sparse_to_dense(AS_CHM_SP__(x), &c), 1);
 }
 
+#endif /* MJ */
+
 SEXP Csparse_to_Tsparse(SEXP x, SEXP tri)
 {
     CHM_SP chxs = AS_CHM_SP__(x);
@@ -310,6 +343,9 @@ SEXP Csparse_to_Tsparse(SEXP x, SEXP tri)
 			       GET_SLOT(x, Matrix_DimNamesSym));
 }
 
+/* MJ: unused */
+#if 0
+
 SEXP Csparse_to_tCsparse(SEXP x, SEXP uplo, SEXP diag)
 {
     CHM_SP chxs = AS_CHM_SP__(x);
@@ -333,6 +369,10 @@ SEXP Csparse_to_tTsparse(SEXP x, SEXP uplo, SEXP diag)
 			       GET_SLOT(x, Matrix_DimNamesSym));
 }
 
+#endif
+
+/* MJ: no longer needed ... prefer R_sparse_as_general() */
+#if 0
 
 SEXP Csparse_symmetric_to_general(SEXP x)
 {
@@ -343,11 +383,16 @@ SEXP Csparse_symmetric_to_general(SEXP x)
     if (!(chx->stype))
 	error(_("Nonsymmetric matrix in Csparse_symmetric_to_general"));
     chgx = cholmod_copy(chx, /* stype: */ 0, chx->xtype, &c);
-    /* xtype: pattern, "real", complex or .. */
     return chm_sparse_to_SEXP(chgx, 1, 0, Rkind, "",
-			      symmetric_DimNames(GET_SLOT(x, Matrix_DimNamesSym)));
+			      get_symmetrized_DimNames(x, -1));
 }
 
+#endif /* MJ */
+
+/* MJ: no longer needed ... prefer R_sparse_force_symmetric() */
+#if 0
+
+// Called from R's forceSymmetricCsparse(), .gC2sC()
 SEXP Csparse_general_to_symmetric(SEXP x, SEXP uplo, SEXP sym_dmns)
 {
     int *adims = INTEGER(GET_SLOT(x, Matrix_DimSym)), n = adims[0];
@@ -356,43 +401,37 @@ SEXP Csparse_general_to_symmetric(SEXP x, SEXP uplo, SEXP sym_dmns)
 	return R_NilValue; /* -Wall */
     }
     CHM_SP chx = AS_CHM_SP__(x), chgx;
-    int uploT = (*CHAR(asChar(uplo)) == 'U') ? 1 : -1;
+    int uploT = (*CHAR(asChar(uplo)) == 'U');
     int Rkind = (chx->xtype != CHOLMOD_PATTERN) ? Real_kind(x) : 0;
     R_CheckStack();
-    chgx = cholmod_copy(chx, /* stype: */ uploT, chx->xtype, &c);
+    chgx = cholmod_copy(chx, /* stype: */ (uploT ? 1 : -1), chx->xtype, &c);
 
-    SEXP dns = GET_SLOT(x, Matrix_DimNamesSym);
-    if(asLogical(sym_dmns))
-	dns = symmetric_DimNames(dns);
-    else if((!isNull(VECTOR_ELT(dns, 0)) &&
-	     !isNull(VECTOR_ELT(dns, 1))) ||
-	    !isNull(getAttrib(dns, R_NamesSymbol))) {
-	/* symmetrize them if both are not NULL
-	 * or names(dimnames(.)) is asymmetric : */
-	dns = PROTECT(duplicate(dns));
-	if(!equal_string_vectors(VECTOR_ELT(dns, 0),
-				 VECTOR_ELT(dns, 1))) {
-	    if(uploT == 1)
-		SET_VECTOR_ELT(dns, 0, VECTOR_ELT(dns,1));
-	    else
-		SET_VECTOR_ELT(dns, 1, VECTOR_ELT(dns,0));
-	}
-	SEXP nms_dns = getAttrib(dns, R_NamesSymbol);
-	if(!isNull(nms_dns) &&  // names(dimnames(.)) :
-	   !R_compute_identical(STRING_ELT(nms_dns, 0),
-				STRING_ELT(nms_dns, 1), 16)) {
-	    if(uploT == 1)
-		SET_STRING_ELT(nms_dns, 0, STRING_ELT(nms_dns,1));
-	    else
-		SET_STRING_ELT(nms_dns, 1, STRING_ELT(nms_dns,0));
-	    setAttrib(dns, R_NamesSymbol, nms_dns);
-	}
-	UNPROTECT(1);
+    SEXP ans, dn = GET_SLOT(x, Matrix_DimNamesSym);
+    int symDmns = asLogical(sym_dmns);
+    /* 3 cases ... 0: do nothing
+                   1: symmetrize
+          NA_LOGICAL: symmetrize if necessary
+    */
+    symDmns = (symDmns == 1 ||
+	       (symDmns != 0 &&
+		((!isNull(VECTOR_ELT(dn, 0)) && !isNull(VECTOR_ELT(dn, 1))) ||
+		 !isNull(getAttrib(dn, R_NamesSymbol)))));
+    if (symDmns) {
+	SEXP newdn = PROTECT(allocVector(VECSXP, 2));
+	symmDN(newdn, dn, -1);
+	dn = newdn;
     }
-    /* xtype: pattern, "real", complex or .. */
-    return chm_sparse_to_SEXP(chgx, 1, 0, Rkind, "", dns);
+    ans = chm_sparse_to_SEXP(chgx, 1, 0, Rkind, "", dn);
+    UNPROTECT(symDmns);
+    return ans;
 }
 
+#endif /* MJ */
+
+/* MJ: no longer needed ... prefer R_sparse_transpose() */
+/* MJ: however, some reverse dependencies built with Matrix < 1.5-0 need it */
+#ifdef Matrix_SupportingCachedMethods
+
 SEXP Csparse_transpose(SEXP x, SEXP tri)
 {
     /* TODO: lgCMatrix & igC* currently go via double prec. cholmod -
@@ -407,19 +446,24 @@ SEXP Csparse_transpose(SEXP x, SEXP tri)
     tmp = VECTOR_ELT(dn, 0);	/* swap the dimnames */
     SET_VECTOR_ELT(dn, 0, VECTOR_ELT(dn, 1));
     SET_VECTOR_ELT(dn, 1, tmp);
-    if(!isNull(tmp = getAttrib(dn, R_NamesSymbol))) { // swap names(dimnames(.)):
+    tmp = PROTECT(getAttrib(dn, R_NamesSymbol));
+    if(!isNull(tmp)) { // swap names(dimnames(.)):
 	SEXP nms_dns = PROTECT(allocVector(VECSXP, 2));
 	SET_VECTOR_ELT(nms_dns, 1, STRING_ELT(tmp, 0));
-        SET_VECTOR_ELT(nms_dns, 0, STRING_ELT(tmp, 1));
+	SET_VECTOR_ELT(nms_dns, 0, STRING_ELT(tmp, 1));
 	setAttrib(dn, R_NamesSymbol, nms_dns);
 	UNPROTECT(1);
     }
-    UNPROTECT(1);
-    return chm_sparse_to_SEXP(chxt, 1, /* SWAP 'uplo' for triangular */
-			      tr ? ((*uplo_P(x) == 'U') ? -1 : 1) : 0,
-			      Rkind, tr ? diag_P(x) : "", dn);
+
+    SEXP ans = chm_sparse_to_SEXP(chxt, 1, /* SWAP 'uplo' for triangular */
+				  tr ? ((*uplo_P(x) == 'U') ? -1 : 1) : 0,
+				  Rkind, tr ? diag_P(x) : "", dn);
+    UNPROTECT(2);
+    return ans;
 }
 
+#endif /* MJ */
+
 /** @brief  A %*% B  - for matrices of class CsparseMatrix (R package "Matrix")
  *
  * @param a
@@ -492,11 +536,20 @@ SEXP Csparse_Csparse_prod(SEXP a, SEXP b, SEXP bool_arith)
 	    else diag[0]= 'N';
 	}
 
+    // establish dimnames --  extra care for *symmetric* Csparse:
+    static const char *valid_sym[] = { MATRIX_VALID_sym_Csparse, "" };
+    Rboolean
+	a_symm = R_check_class_etc(a, valid_sym) >= 0,
+	b_symm = R_check_class_etc(b, valid_sym) >= 0;
     SEXP dn = PROTECT(allocVector(VECSXP, 2));
-    SET_VECTOR_ELT(dn, 0,	/* establish dimnames */
-		   duplicate(VECTOR_ELT(GET_SLOT(a, Matrix_DimNamesSym), 0)));
+    SET_VECTOR_ELT(dn, 0,
+		   duplicate(VECTOR_ELT(a_symm ?
+					get_symmetrized_DimNames(a, -1)
+					: GET_SLOT(a, Matrix_DimNamesSym), 0)));
     SET_VECTOR_ELT(dn, 1,
-		   duplicate(VECTOR_ELT(GET_SLOT(b, Matrix_DimNamesSym), 1)));
+		   duplicate(VECTOR_ELT(b_symm
+					? get_symmetrized_DimNames(b, -1)
+					: GET_SLOT(b, Matrix_DimNamesSym), 1)));
     UNPROTECT(nprot);
     return chm_sparse_to_SEXP(chc, 1, uploT, /*Rkind*/0, diag, dn);
 }
@@ -560,11 +613,11 @@ SEXP Csparse_Csparse_crossprod(SEXP a, SEXP b, SEXP trans, SEXP bool_arith)
     cholmod_free_sparse(&chTr, &c);
 
     /* Preserve triangularity and unit-triangularity if appropriate;
-     * see Csparse_Csparse_prod() for comments */
+     * see Csparse_Csparse_prod() for more comments */
     if(R_check_class_etc(a, valid_tri) >= 0 &&
        R_check_class_etc(b, valid_tri) >= 0)
 	if(*uplo_P(a) != *uplo_P(b)) { /* one 'U', the other 'L' */
-	    uploT = (*uplo_P(b) == 'U') ? 1 : -1;
+	    uploT = (*uplo_P(b) == (tr ? 'L' : 'U')) ? 1 : -1;
 	    if(*diag_P(a) == 'U' && *diag_P(b) == 'U') { /* return UNIT-triag. */
 		chm_diagN2U(chc, uploT, /* do_realloc */ FALSE);
 		diag[0]= 'U';
@@ -616,7 +669,8 @@ SEXP Csp_dense_products(SEXP a, SEXP b,
 			Rboolean transp_a, Rboolean transp_b, Rboolean transp_ans)
 {
     CHM_SP cha = AS_CHM_SP(a);
-    int a_nc = transp_a ? cha->nrow : cha->ncol,
+    size_t
+	a_nc = transp_a ? cha->nrow : cha->ncol,
 	a_nr = transp_a ? cha->ncol : cha->nrow;
     Rboolean
 	maybe_transp_b = (a_nc == 1),
@@ -627,29 +681,26 @@ SEXP Csp_dense_products(SEXP a, SEXP b,
        according to the general R philosophy of treating vectors in matrix products.
     */
 
-    /* repeating a "cheap part" of  mMatrix_as_dgeMatrix2(b, .)  to see if
-     * we have a vector that we might 'transpose_if_vector' : */
-    static const char *valid[] = {"_NOT_A_CLASS_", MATRIX_VALID_ddense, ""};
-    /* int ctype = R_check_class_etc(b, valid);
-     * if (ctype > 0)   /.* a ddenseMatrix object */
-    if (R_check_class_etc(b, valid) < 0) {
-	// not a ddenseM*:  is.matrix() or vector:
+    /* repeating a "cheap part" of dense_as_general() 
+       to see if we have a vector that we might 'transpose_if_vector' : */
+    static const char *valid[] = {MATRIX_VALID_ddense, ""};
+    if (R_check_class_etc(b, valid) < 0)
+	// _not_ a ddenseM* or a ddiM*:  is.matrix() or vector:
 	b_is_vector = !isMatrix(b);
-    }
 
     if(b_is_vector) {
 	/* determine *if* we want/need to transpose at all:
 	 * if (length(b) == ncol(A)) have match: use dim = c(n, 1) (<=> do *not* transp);
 	 *  otherwise, try to transpose: ok  if (ncol(A) == 1) [see also above]:  */
-	maybe_transp_b = (LENGTH(b) != a_nc);
-	// Here, we transpose already in mMatrix_as_dge*()  ==> don't do it later:
+	maybe_transp_b = (XLENGTH(b) != a_nc);
+	// Here, we transpose already in dense_as_ge*()  ==> don't do it later:
 	transp_b = FALSE;
     }
-    SEXP b_M = PROTECT(mMatrix_as_dgeMatrix2(b, maybe_transp_b));
+    SEXP b_M = PROTECT(dense_as_general(b, 'd', 2, maybe_transp_b));
 
     CHM_DN chb = AS_CHM_DN(b_M), b_t;
     R_CheckStack();
-    int ncol_b;
+    size_t ncol_b;
     if(transp_b) { // transpose b:
 	b_t = cholmod_allocate_dense(chb->ncol, chb->nrow, chb->ncol, chb->xtype, &c);
 	chm_transpose_dense(b_t, chb);
@@ -665,10 +716,10 @@ SEXP Csp_dense_products(SEXP a, SEXP b,
     if(cha->xtype == CHOLMOD_PATTERN) {
 	/* warning(_("Csparse_dense_prod(): cholmod_sdmult() not yet implemented for pattern./ ngCMatrix" */
 	/* 	  " --> slightly inefficient coercion")); */
-
 	// This *fails* to produce a CHOLMOD_REAL ..
-	// CHM_SP chd = cholmod_l_copy(cha, cha->stype, CHOLMOD_REAL, &c);
-	// --> use our Matrix-classes
+	// CHM_SP chd = cholmod_l_copy(cha, cha->stype, CHOLMOD_REAL, &cl);
+
+	// --> use our Matrix-classes: they work:
 	SEXP da = PROTECT(nz2Csparse(a, x_double)); nprot++;
 	cha = AS_CHM_SP(da);
     }
@@ -753,12 +804,12 @@ SEXP Csparse_crossprod(SEXP x, SEXP trans, SEXP triplet, SEXP bool_arith)
 
     if (!tr) chxt = cholmod_transpose(chx, chx->xtype, &c);
 
-    if (x_is_sym) // cholmod_aat() does not like symmetric
-	chxc = cholmod_copy(tr ? chx : chxt, /* stype: */ 0,
-			    chx->xtype, &c);
+    // cholmod_aat() does not like symmetric
+    chxc = x_is_sym ? cholmod_copy(tr ? chx : chxt, /* stype: */ 0, chx->xtype, &c) : NULL;
     // CHOLMOD/Core/cholmod_aat.c :
     chcp = cholmod_aat(x_is_sym ? chxc : (tr ? chx : chxt),
 		       (int *) NULL, 0, /* mode: */ chx->xtype, &c);
+    if (chxc) cholmod_free_sparse(&chxc, &c);
     if(!chcp) {
 	UNPROTECT(1);
 	error(_("Csparse_crossprod(): error return from cholmod_aat()"));
@@ -804,16 +855,21 @@ SEXP Csparse_horzcat(SEXP x, SEXP y)
 #define CSPARSE_CAT(_KIND_)						\
     CHM_SP chx = AS_CHM_SP__(x), chy = AS_CHM_SP__(y);			\
     R_CheckStack();							\
-    int Rk_x = (chx->xtype != CHOLMOD_PATTERN) ? Real_kind(x) : -3,	\
-	Rk_y = (chy->xtype != CHOLMOD_PATTERN) ? Real_kind(y) : -3, Rkind; \
-    if(Rk_x == -3 || Rk_y == -3) { /* at least one of them is patter"n" */ \
-	if(Rk_x == -3 && Rk_y == -3) { /* fine */			\
+    void* chx_x = chx->x;						\
+    void* chx_z = chx->z;						\
+    void* chy_x = chy->x;						\
+    void* chy_z = chy->z;						\
+    int Rk_x = (chx->xtype != CHOLMOD_PATTERN) ? Real_kind(x) : x_pattern, \
+	Rk_y = (chy->xtype != CHOLMOD_PATTERN) ? Real_kind(y) : x_pattern, Rkind; \
+    if(Rk_x == x_pattern || Rk_y == x_pattern) { /* at least one of them is patter"n" */ \
+	if(Rk_x == x_pattern && Rk_y == x_pattern) { /* fine */		\
 	} else { /* only one is a patter"n"				\
-		  * "Bug" in cholmod_horzcat()/vertcat(): returns patter"n" matrix if one of them is */	\
+		  * "Bug" in cholmod_horzcat()/vertcat():               \
+		  * returns patter"n" matrix if one of them is */	\
 	    Rboolean ok;						\
-	    if(Rk_x == -3) {						\
+	    if(Rk_x == x_pattern) {					\
 		ok = chm_MOD_xtype(CHOLMOD_REAL, chx, &c); Rk_x = 0;	\
-	    } else if(Rk_y == -3) {					\
+	    } else if(Rk_y == x_pattern) {				\
 		ok = chm_MOD_xtype(CHOLMOD_REAL, chy, &c); Rk_y = 0;	\
 	    } else							\
 		error(_("Impossible Rk_x/Rk_y in Csparse_%s(), please report"), _KIND_); \
@@ -827,8 +883,23 @@ SEXP Csparse_horzcat(SEXP x, SEXP y)
     CSPARSE_CAT("horzcat");
     // TODO: currently drops dimnames - and we fix at R level;
 
-    return chm_sparse_to_SEXP(cholmod_horzcat(chx, chy, 1, &c),
-			      1, 0, Rkind, "", R_NilValue);
+    SEXP retval = PROTECT(
+	chm_sparse_to_SEXP(cholmod_horzcat(chx, chy, 1, &c),
+			   1, 0, Rkind, "", R_NilValue));
+/* AS_CHM_SP(x) fills result with points to R-allocated memory but
+   chm_MOD_xtype can change ->x and ->z to cholmod_alloc'ed memory.
+   The former needs no freeing but the latter does.
+   The first 2 arguments to cholmod_free should contain the number
+   and size of things being freed, but lying about that is sort of ok. */
+#define CSPARSE_CAT_CLEANUP					\
+    if (chx_x != chx->x) cholmod_free(0, 0, chx->x, &c);	\
+    if (chx_z != chx->z) cholmod_free(0, 0, chx->z, &c);	\
+    if (chy_x != chy->x) cholmod_free(0, 0, chy->x, &c);	\
+    if (chy_z != chy->z) cholmod_free(0, 0, chy->z, &c);	\
+    UNPROTECT(1);
+
+    CSPARSE_CAT_CLEANUP;
+    return retval;
 }
 
 /** @brief Vertical Concatenation -  rbind( <Csparse>,  <Csparse>)
@@ -838,10 +909,17 @@ SEXP Csparse_vertcat(SEXP x, SEXP y)
     CSPARSE_CAT("vertcat");
     // TODO: currently drops dimnames - and we fix at R level;
 
-    return chm_sparse_to_SEXP(cholmod_vertcat(chx, chy, 1, &c),
-			      1, 0, Rkind, "", R_NilValue);
+    SEXP retval  = PROTECT(
+	chm_sparse_to_SEXP(cholmod_vertcat(chx, chy, 1, &c),
+			   1, 0, Rkind, "", R_NilValue));
+    CSPARSE_CAT_CLEANUP;
+    return retval;
 }
 
+/* MJ: no longer needed ... prefer R_sparse_band() */
+/* MJ: however, some reverse dependencies built with Matrix < 1.5-0 need it */
+#ifdef Matrix_SupportingCachedMethods
+
 SEXP Csparse_band(SEXP x, SEXP k1, SEXP k2)
 {
     CHM_SP chx = AS_CHM_SP__(x);
@@ -849,10 +927,12 @@ SEXP Csparse_band(SEXP x, SEXP k1, SEXP k2)
     CHM_SP ans = cholmod_band(chx, asInteger(k1), asInteger(k2), chx->xtype, &c);
     R_CheckStack();
 
-    return chm_sparse_to_SEXP(ans, 1, 0, Rkind, "",
+    return chm_sparse_to_SEXP(ans, 1, /* uploT = */ 0, Rkind, "",
 			      GET_SLOT(x, Matrix_DimNamesSym));
 }
 
+#endif /* MJ */
+
 SEXP Csparse_diagU2N(SEXP x)
 {
     const char *cl = class_P(x);
@@ -926,6 +1006,17 @@ SEXP Csparse_submatrix(SEXP x, SEXP i, SEXP j)
     if (csize >= 0 && !isInteger(j))
 	error(_("Index j must be NULL or integer"));
 
+    /* Must treat 'NA's in i[] and j[] here -- they are *not* treated by Cholmod!
+     * haveNA := ...
+       if(haveNA) {
+	 a. i = removeNA(i); j =removeNA(j), and remember where they were
+	 b. ans = CHM_SUB(.., i, j)
+	 c. add NA rows and/or columns to 'ans' according to
+	    place of NA's in i and/or j.
+       } else {
+	 ans = CHM_SUB(.....)  // == current code
+       }
+     */
 #define CHM_SUB(_M_, _i_, _j_)					\
     cholmod_submatrix(_M_,					\
 		      (rsize < 0) ? NULL : INTEGER(_i_), rsize,	\
@@ -935,8 +1026,8 @@ SEXP Csparse_submatrix(SEXP x, SEXP i, SEXP j)
     if (!chx->stype) {/* non-symmetric Matrix */
 	ans = CHM_SUB(chx, i, j);
     }
-    else {
-	/* for now, cholmod_submatrix() only accepts "generalMatrix" */
+    else { /* symmetric : "dsCMatrix";
+	      currently, cholmod_submatrix() only accepts "generalMatrix" */
 	CHM_SP tmp = cholmod_copy(chx, /* stype: */ 0, chx->xtype, &c);
 	ans = CHM_SUB(tmp, i, j);
 	cholmod_free_sparse(&tmp, &c);
@@ -948,6 +1039,7 @@ SEXP Csparse_submatrix(SEXP x, SEXP i, SEXP j)
     /* 	dn = PROTECT(allocVector(VECSXP, 2)); */
     return chm_sparse_to_SEXP(ans, 1, 0, Rkind, "", /* dimnames: */ R_NilValue);
 }
+#undef CHM_SUB
 
 #define _d_Csp_
 #include "t_Csparse_subassign.c"
@@ -994,7 +1086,7 @@ SEXP Csparse_MatrixMarket(SEXP x, SEXP fname)
  * @return  a SEXP, either a (double) number or a length n-vector of diagonal entries
  */
 SEXP diag_tC_ptr(int n, int *x_p, double *x_x, Rboolean is_U, int *perm,
-/*                                ^^^^^^ FIXME[Generalize] to int / ... */
+/*                                ^^^^^^ FIXME[Generalize] to int / ... -- via x_slot_kind ? */
 		 SEXP resultKind)
 {
     const char* res_ch = CHAR(STRING_ELT(resultKind,0));
@@ -1107,7 +1199,6 @@ SEXP diag_tC_ptr(int n, int *x_p, double *x_x, Rboolean is_U, int *perm,
  */
 SEXP diag_tC(SEXP obj, SEXP resultKind)
 {
-
     SEXP
 	pslot = GET_SLOT(obj, Matrix_pSym),
 	xslot = GET_SLOT(obj, Matrix_xSym);
@@ -1149,8 +1240,7 @@ SEXP create_Csparse(char* cls, int* i, int* j, int* p, int np,
 		    int index1)
 {
     SEXP ans;
-    int *ij = (int*)NULL, *tri, *trj,
-	mi, mj, mp, nrow = -1, ncol = -1;
+    int *ij = (int*)NULL, *tri, *trj, nrow = -1, ncol = -1;
     int xtype = -1;		/* -Wall */
     CHM_TR T;
     CHM_SP A;
@@ -1158,9 +1248,10 @@ SEXP create_Csparse(char* cls, int* i, int* j, int* p, int np,
     if (np < 0 || nnz < 0)
 	error(_("negative vector lengths not allowed: np = %d, nnz = %d"),
 	      np, nnz);
-    if (1 != ((mi = (i == (int*)NULL)) +
-	      (mj = (j == (int*)NULL)) +
-	      (mp = (p == (int*)NULL))))
+    int mi = (i == (int*)NULL), // := missing 'i'
+	mj = (j == (int*)NULL), // := missing 'j'
+	mp = (p == (int*)NULL); // := missing 'p'
+    if ((mi + mj + mp) != 1)
 	error(_("exactly 1 of 'i', 'j' or 'p' must be NULL"));
     if (mp) {
 	if (np) error(_("np = %d, must be zero when p is NULL"), np);
@@ -1172,7 +1263,7 @@ SEXP create_Csparse(char* cls, int* i, int* j, int* p, int np,
 		    error(_("p must be non-decreasing"));
 	    if (p[np] != nnz)
 		error("p[np] = %d != nnz = %d", p[np], nnz);
-	    ij = Calloc(nnz, int);
+	    ij = R_Calloc(nnz, int);
 	    if (mi) {
 		i = ij;
 		nrow = np;
@@ -1211,7 +1302,7 @@ SEXP create_Csparse(char* cls, int* i, int* j, int* p, int np,
     /* check the class name */
     if (strlen(cls) != 8)
 	error(_("strlen of cls argument = %d, should be 8"), strlen(cls));
-    if (!strcmp(cls + 2, "CMatrix"))
+    if (strcmp(cls + 2, "CMatrix"))
 	error(_("cls = \"%s\" does not end in \"CMatrix\""), cls);
     switch(cls[0]) {
     case 'd':
@@ -1240,15 +1331,16 @@ SEXP create_Csparse(char* cls, int* i, int* j, int* p, int np,
     A = cholmod_triplet_to_sparse(T, nnz, &c);
     cholmod_free_triplet(&T, &c);
     /* copy the information to the SEXP */
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS(cls)));
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(cls));
 // FIXME: This has been copied from chm_sparse_to_SEXP in  chm_common.c
     /* allocate and copy common slots */
-    nnz = cholmod_nnz(A, &c);
+    nnz = (int) cholmod_nnz(A, &c);
     dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
-    dims[0] = A->nrow; dims[1] = A->ncol;
+    dims[0] = (int) A->nrow;
+    dims[1] = (int) A->ncol;
     Memcpy(INTEGER(ALLOC_SLOT(ans, Matrix_pSym, INTSXP, A->ncol + 1)), (int*)A->p, A->ncol + 1);
     Memcpy(INTEGER(ALLOC_SLOT(ans, Matrix_iSym, INTSXP, nnz)), (int*)A->i, nnz);
-    switch(cls[1]) {
+    switch(cls[0]) {
     case 'd':
 	Memcpy(REAL(ALLOC_SLOT(ans, Matrix_xSym, REALSXP, nnz)), (double*)A->x, nnz);
 	break;
@@ -1260,3 +1352,197 @@ SEXP create_Csparse(char* cls, int* i, int* j, int* p, int np,
     UNPROTECT(1);
     return ans;
 }
+
+/* MJ: no longer needed ... prefer R_dense_as_sparse() */
+#if 0
+
+/**
+ * Create a Csparse matrix object from a traditional R matrix
+ *
+ * @param x   traditional R matrix (numeric, logical, ...)
+ * @param cls class (a string), currently must be  "..CMatrix"
+ *
+ * @return an SEXP of a class inheriting from CsparseMatrix.
+ */
+SEXP matrix_to_Csparse(SEXP x, SEXP cls)
+{
+    if (!isMatrix(x))
+	error(_("%s must be (traditional R) matrix"), "'x'");
+    SEXP d_x  = getAttrib(x, R_DimSymbol),
+	dn_x  = getAttrib(x, R_DimNamesSymbol);
+    int nr = INTEGER(d_x)[0],
+	nc = INTEGER(d_x)[1];
+
+    if (!(isString(cls) && LENGTH(cls) == 1))
+	error(_("%s must be character string"), "'cls'");
+    R_xlen_t ii, n = XLENGTH(x);
+    int xtype = -1;
+    if (n != ((R_xlen_t) nr) * nc)
+	error(_("nrow * ncol = %d * %d must equal length(x) = %ld"), nr, nc, n);
+
+    const char *ccls = CHAR(STRING_ELT(cls, 0));
+    if (strlen(ccls) != 9)
+	error(_("strlen of cls argument = %d, should be 9"), strlen(ccls));
+    if (strcmp(ccls + 2, "CMatrix"))
+	error(_("cls = \"%s\" does not end in \"CMatrix\""), ccls);
+    switch(ccls[0]) {
+    case 'd':
+    case 'l':
+	xtype = CHOLMOD_REAL;
+    break;
+    case 'n':
+	xtype = CHOLMOD_PATTERN;
+	break;
+    default:
+	error(_("cls = \"%s\" must begin with 'd', 'l' or 'n' for now"), ccls);
+    }
+    /* if (ccls[1] != 'g') */
+    /* 	error(_("Only 'g'eneral sparse matrix types allowed")); */
+
+    SEXP ans = PROTECT(NEW_OBJECT_OF_CLASS(ccls));
+    SET_SLOT(ans, Matrix_DimSym, d_x);
+    SET_SLOT(ans, Matrix_DimNamesSym, (!isNull(dn_x) && LENGTH(dn_x) == 2)
+	     ? duplicate(dn_x)
+	     : allocVector(VECSXP, 2));
+
+    int nz = 0, // current number of nonzero entries
+	nnz = MAXOF(256, MAXOF(nr,nc));/* nnz := final number of nonzero entries, yet unknown;
+					   -- must start with guess and then grow */
+    int *rp = INTEGER(ALLOC_SLOT(ans, Matrix_pSym, INTSXP, nc + 1)),
+	*ri = R_Calloc(nnz, int); // to become i slot -- of not-yet-known length nnz
+
+    rp[0] = 0; // always
+
+    switch(TYPEOF(x)) {
+    case LGLSXP: {
+	if(xtype == CHOLMOD_PATTERN) {
+#         define _PATTERN_x
+#         include "t_matrix_to_Csp.c"
+	} else {
+#         define _LOGICAL_x
+#         include "t_matrix_to_Csp.c"
+	}
+	break;
+    }
+    case REALSXP: {
+#       define _DOUBLE_x
+#       include "t_matrix_to_Csp.c"
+	break;
+    }
+/* case INTSXP: we would have to use
+	x = coerceVector(x, REALSXP));
+   and then fall through to REALSXP case, but we must *not* modify 'x' here
+   FIXME: use a macro or (inline?) function with argument (y), where
+   -----  SEXP y = PROTECT(coerceVector(x, REALSXP))
+
+   ==> give error in INTSXP case, so caller (in R) must set  storage.mode(x) <- "double"
+*/
+#ifdef _USING_INTEGER_NOT_READY__
+    case INTSXP: {
+#       define _INTEGER_x
+#       include "t_matrix_to_Csp.c"
+	break;
+    }
+#endif
+#ifdef _USING_COMPLEX_NOT_READY__
+    case CPLXSXP: {
+#       define _COMPLEX_x
+#       include "t_matrix_to_Csp.c"
+	break;
+    }
+#endif
+    default:
+	error(_("%s must be a logical or double vector"), "'x'");
+	break;
+    }
+
+    Memcpy(INTEGER(ALLOC_SLOT(ans, Matrix_iSym,  INTSXP, nnz)), ri, nnz);
+    R_Free(ri);
+
+    UNPROTECT(1);
+    return ans;
+}
+
+#endif /* MJ */
+
+// seed will *not* be used unless it's -1 (inverse perm.) or  0 ("no" / identity) perm.
+static csd* Csparse_dmperm_raw(SEXP mat, SEXP seed)
+{
+    mat = PROTECT(duplicate(mat));
+    CSP matx = AS_CSP__(mat); /* m x n ; compressed column, *double* 'x' or none */
+    int iseed = asInteger(seed);
+    R_CheckStack();
+    UNPROTECT(1);
+    return cs_dmperm(matx, iseed); // -> ./cs.c
+}
+
+/* NB:  cs.h  defines the 'csd' struct as  (NB: csi :== int in  Matrix, for now)
+
+   typedef struct cs_dmperm_results    // cs_dmperm or cs_scc output
+   {
+   csi *p ;        // size m, row permutation
+   csi *q ;        // size n, column permutation
+   csi *r ;        // size nb+1, block k is rows r[k] to r[k+1]-1 in A(p,q)
+   csi *s ;        // size nb+1, block k is cols s[k] to s[k+1]-1 in A(p,q)
+   csi nb ;        // # of blocks in fine dmperm decomposition
+   csi rr [5] ;    // coarse row decomposition
+   csi cc [5] ;    // coarse column decomposition
+   } csd ;
+*/
+
+/* MM: should allow to return the full info above
+   (Timothy Davis, p.126, explains why it's interesting ..) */
+
+/* Here, return the full *named* list to R */
+SEXP Csparse_dmperm(SEXP mat, SEXP seed, SEXP nAns) {
+    csd *DMp = Csparse_dmperm_raw(mat, seed);
+    if(DMp == NULL) // "failure" in cs_dmperm()
+	return(R_NilValue);
+    int *dims = INTEGER(GET_SLOT(mat, Matrix_DimSym)),
+	m = dims[0],
+	n = dims[1],
+	n_ans = asInteger(nAns),
+	nb = DMp->nb;
+
+    SEXP nms = PROTECT(allocVector(STRSXP, n_ans));
+    SEXP ans = PROTECT(allocVector(VECSXP, n_ans));
+    R_CheckStack();
+    int *ip;
+    /* p : */SET_STRING_ELT(nms, 0, mkChar("p"));
+             SET_VECTOR_ELT(ans, 0, allocVector(INTSXP, m));
+    ip = INTEGER(VECTOR_ELT(ans, 0));
+    /* 0-based permutation:
+     * Memcpy(ip , (int*)(DMp->p), m); */
+    // 1-based permutation:
+    for(int i=0; i < m; i++) ip[i] = DMp->p[i] + 1;
+
+    /* q : */SET_STRING_ELT(nms, 1, mkChar("q"));
+             SET_VECTOR_ELT(ans, 1, allocVector(INTSXP, n));
+    ip = INTEGER(VECTOR_ELT(ans, 1));
+    /* 0-based permutation:
+     * Memcpy(ip , (int*)(DMp->q), m); */
+    // 1-based permutation:
+    for(int i=0; i < n; i++) ip[i] = DMp->q[i] + 1;
+
+    if(n_ans > 2) {
+      /* r : */  SET_STRING_ELT(nms, 2, mkChar("r"));
+		 SET_VECTOR_ELT(ans, 2, allocVector(INTSXP, nb+1));
+      Memcpy(INTEGER(VECTOR_ELT(ans, 2)), (int*)(DMp->r),   nb+1);
+
+      /* s : */  SET_STRING_ELT(nms, 3, mkChar("s"));
+		 SET_VECTOR_ELT(ans, 3, allocVector(INTSXP, nb+1));
+      Memcpy(INTEGER(VECTOR_ELT(ans, 3)), (int*)(DMp->s),   nb+1);
+      if(n_ans > 4) {
+	/* rr5 :*/ SET_STRING_ELT(nms, 4, mkChar("rr5"));
+		   SET_VECTOR_ELT(ans, 4, allocVector(INTSXP, 5));
+	Memcpy(INTEGER(VECTOR_ELT(ans, 4)), (int*)(DMp->rr),  5);
+
+	/* cc5 :*/ SET_STRING_ELT(nms, 5, mkChar("cc5"));
+		   SET_VECTOR_ELT(ans, 5, allocVector(INTSXP, 5));
+	Memcpy(INTEGER(VECTOR_ELT(ans, 5)), (int*)(DMp->cc),  5);
+      }
+    }
+    setAttrib(ans, R_NamesSymbol, nms);
+    UNPROTECT(2);
+    return ans;
+}
