@@ -1,20 +1,18 @@
-SEXP
-    Matrix_betaSym,
-    Matrix_DimNamesSym,
+SEXP Matrix_DimNamesSym,
     Matrix_DimSym,
+    Matrix_LSym,
+    Matrix_RSym,
+    Matrix_USym,
+    Matrix_VSym,
+    Matrix_betaSym,
     Matrix_diagSym,
     Matrix_factorSym,
     Matrix_iSym,
     Matrix_jSym,
     Matrix_lengthSym,
-    Matrix_LSym, Matrix_RSym, Matrix_USym,
     Matrix_pSym,
     Matrix_permSym,
     Matrix_uploSym,
-    Matrix_VSym,
     Matrix_xSym,
 
-    Matrix_NS;/* the Matrix Namespace ('environment') */
-
-
-
+    Matrix_NS; /* the 'Matrix' namespace ("environment") */
