@@ -417,7 +417,7 @@ DBFOpen( const char * pszFilename, const char * pszAccess )
     psDBF->pszHeader = (char *) pabyBuf;
 
     fseek( psDBF->fp, 32, 0 );
-    if( fread( pabyBuf, nHeadLen-32, 1, psDBF->fp ) != 1 )
+    if( nHeadLen <= 32 || fread( pabyBuf, nHeadLen-32, 1, psDBF->fp ) != 1 )
     {
 	fclose( psDBF->fp );
 	free( pabyBuf );
