@@ -1,36 +1,6 @@
 #include "dgeMatrix.h"
-
-SEXP dMatrix_validate(SEXP obj)
-{
-    SEXP x = GET_SLOT(obj, Matrix_xSym),
-	Dim = GET_SLOT(obj, Matrix_DimSym);
-    int m, n;
-
-    if (length(Dim) != 2)
-	return mkString(_("Dim slot must have length 2"));
-    m = INTEGER(Dim)[0]; n = INTEGER(Dim)[1];
-    if (m < 0 || n < 0)
-	return mkString(dngettext("Matrix",
-				  "Negative value in Dim",
-				  "Negative values in Dim",
-				  (m*n > 0) ? 2 : 1));
-    if (!isReal(x))
-	return mkString(_("x slot must be numeric \"double\""));
-    return ScalarLogical(1);
-}
-
-SEXP dgeMatrix_validate(SEXP obj)
-{
-    SEXP val,
-	fact = GET_SLOT(obj, Matrix_factorSym);
-
-    if (isString(val = dense_nonpacked_validate(obj)))
-	return(val);
-
-    if (length(fact) > 0 && getAttrib(fact, R_NamesSymbol) == R_NilValue)
-	return mkString(_("factors slot must be named list"));
-    return ScalarLogical(1);
-}
+// -> Mutils.h  etc
+#include <float.h>
 
 static
 double get_norm(SEXP obj, const char *typstr)
@@ -48,7 +18,7 @@ double get_norm(SEXP obj, const char *typstr)
 	}
 	return F77_CALL(dlange)(typstr, dims, dims+1,
 				REAL(GET_SLOT(obj, Matrix_xSym)),
-				dims, work);
+				dims, work FCONE);
     }
 }
 
@@ -74,139 +44,316 @@ SEXP dgeMatrix_rcond(SEXP obj, SEXP type)
 		     dims, REAL(GET_SLOT(LU, Matrix_xSym)),
 		     dims, &anorm, &rcond,
 		     (double *) R_alloc(4*dims[0], sizeof(double)),
-		     (int *) R_alloc(dims[0], sizeof(int)), &info);
+		     (int *) R_alloc(dims[0], sizeof(int)), &info FCONE);
     UNPROTECT(1);
     return ScalarReal(rcond);
 }
 
 SEXP dgeMatrix_crossprod(SEXP x, SEXP trans)
 {
-    int tr = asLogical(trans);/* trans=TRUE: tcrossprod(x) */
-    SEXP val = PROTECT(NEW_OBJECT(MAKE_CLASS("dpoMatrix"))),
-	nms = VECTOR_ELT(GET_SLOT(x, Matrix_DimNamesSym), tr ? 0 : 1),
-	vDnms = ALLOC_SLOT(val, Matrix_DimNamesSym, VECSXP, 2);
-    int *Dims = INTEGER(GET_SLOT(x, Matrix_DimSym)),
-	*vDims = INTEGER(ALLOC_SLOT(val, Matrix_DimSym, INTSXP, 2));
-    int k = tr ? Dims[1] : Dims[0], n = tr ? Dims[0] : Dims[1];
-    double *vx = REAL(ALLOC_SLOT(val, Matrix_xSym, REALSXP, n * n)),
-	one = 1.0, zero = 0.0;
+#define DGE_CROSS_1							\
+    int tr = asLogical(trans);/* trans=TRUE: tcrossprod(x) */		\
+    SEXP val = PROTECT(NEW_OBJECT_OF_CLASS("dpoMatrix")),		\
+	vDnms = PROTECT(ALLOC_SLOT(val, Matrix_DimNamesSym, VECSXP, 2)),\
+	nms  = VECTOR_ELT(GET_SLOT(x, Matrix_DimNamesSym), tr ? 0 : 1);	\
+    int *Dims = INTEGER(GET_SLOT(x, Matrix_DimSym)),			\
+	*vDims = INTEGER(ALLOC_SLOT(val, Matrix_DimSym, INTSXP, 2));	\
+    int k = tr ? Dims[1] : Dims[0],					\
+	n = tr ? Dims[0] : Dims[1];					\
+    R_xlen_t n_ = n, n2 = n_ * n_;					\
+    double *vx = REAL(ALLOC_SLOT(val, Matrix_xSym, REALSXP, n2)),	\
+	one = 1.0, zero = 0.0;						\
+    									\
+    Memzero(vx, n2);							\
+    SET_SLOT(val, Matrix_uploSym, mkString("U"));			\
+    ALLOC_SLOT(val, Matrix_factorSym, VECSXP, 0);			\
+    vDims[0] = vDims[1] = n;						\
+    SET_VECTOR_ELT(vDnms, 0, duplicate(nms));				\
+    SET_VECTOR_ELT(vDnms, 1, duplicate(nms))
 
-    AZERO(vx, n * n);
-    SET_SLOT(val, Matrix_uploSym, mkString("U"));
-    ALLOC_SLOT(val, Matrix_factorSym, VECSXP, 0);
-    vDims[0] = vDims[1] = n;
-    SET_VECTOR_ELT(vDnms, 0, duplicate(nms));
-    SET_VECTOR_ELT(vDnms, 1, duplicate(nms));
-    if(n)
-    F77_CALL(dsyrk)("U", tr ? "N" : "T", &n, &k,
-		    &one, REAL(GET_SLOT(x, Matrix_xSym)), Dims,
-		    &zero, vx, &n);
-    SET_SLOT(val, Matrix_factorSym, allocVector(VECSXP, 0));
+#define DGE_CROSS_DO(_X_X_)					\
+    if(n)							\
+	F77_CALL(dsyrk)("U", tr ? "N" : "T", &n, &k, &one,	\
+			_X_X_, Dims, &zero, vx, &n FCONE FCONE);	\
+    UNPROTECT(2);						\
+    return val
+
+    DGE_CROSS_1;
+    DGE_CROSS_DO(REAL(GET_SLOT(x, Matrix_xSym)));
+}
+
+
+double* gematrix_real_x(SEXP x, int nn) {
+    if(class_P(x)[0] == 'd') // <<- FIXME: use R_check_class_etc(x, valid)  !!!
+	return REAL(GET_SLOT(x, Matrix_xSym));
+#ifdef _potentically_more_efficient_but_not_working
+    // else : 'l' or 'n' (for now !!)
+    int *xi = INTEGER(GET_SLOT(x, Matrix_xSym));
+    double *x_x;
+    Calloc_or_Alloca_TO(x_x, nn, double);
+    for(int i=0; i < nn; i++)
+	x_x[i] = (double) xi[i];
+
+    // FIXME: this is not possible either; the *caller* would have to R_Free(.)
+    Free_FROM(x_x, nn);
+#else
+    // ideally should be PROTECT()ed ==> make sure R does not run gc() now!
+    double *x_x = REAL(coerceVector(GET_SLOT(x, Matrix_xSym), REALSXP));
+#endif
+    return x_x;
+}
+
+//!  As  dgeMatrix_crossprod(), but x can be [dln]geMatrix
+SEXP _geMatrix_crossprod(SEXP x, SEXP trans)
+{
+    DGE_CROSS_1;
+    double *x_x = gematrix_real_x(x, k * n_);
+    DGE_CROSS_DO(x_x);
+}
+
+SEXP geMatrix_crossprod(SEXP x, SEXP trans)
+{
+    SEXP y = PROTECT(dense_as_general(x, '.', 2, 0)),
+	val = _geMatrix_crossprod(y, trans);
     UNPROTECT(1);
     return val;
 }
 
 SEXP dgeMatrix_dgeMatrix_crossprod(SEXP x, SEXP y, SEXP trans)
 {
-    int tr = asLogical(trans);/* trans=TRUE: tcrossprod(x,y) */
-    SEXP val = PROTECT(NEW_OBJECT(MAKE_CLASS("dgeMatrix"))),
-	dn = PROTECT(allocVector(VECSXP, 2));
-    int *xDims = INTEGER(GET_SLOT(x, Matrix_DimSym)),
-	*yDims = INTEGER(GET_SLOT(y, Matrix_DimSym)),
-	*vDims;
-    int m  = xDims[!tr],  n = yDims[!tr];/* -> result dim */
-    int xd = xDims[ tr], yd = yDims[ tr];/* the conformable dims */
-    double one = 1.0, zero = 0.0;
+#define DGE_DGE_CROSS_1							\
+    int tr = asLogical(trans);/* trans=TRUE: tcrossprod(x,y) */		\
+    SEXP val = PROTECT(NEW_OBJECT_OF_CLASS("dgeMatrix")),		\
+	dn = PROTECT(allocVector(VECSXP, 2));				\
+    int *xDims = INTEGER(GET_SLOT(x, Matrix_DimSym)),			\
+	*yDims = INTEGER(GET_SLOT(y, Matrix_DimSym)),			\
+	*vDims;								\
+    int m  = xDims[!tr],  n = yDims[!tr];/* -> result dim */		\
+    int xd = xDims[ tr], yd = yDims[ tr];/* the conformable dims */	\
+    double one = 1.0, zero = 0.0;					\
+									\
+    if (xd != yd)							\
+	error(_("Dimensions of x and y are not compatible for %s"),	\
+	      tr ? "tcrossprod" : "crossprod");				\
+    SET_SLOT(val, Matrix_factorSym, allocVector(VECSXP, 0));		\
+    /* establish dimnames */						\
+    SET_VECTOR_ELT(dn, 0,						\
+		   duplicate(VECTOR_ELT(GET_SLOT(x, Matrix_DimNamesSym), \
+					tr ? 0 : 1)));			\
+    SET_VECTOR_ELT(dn, 1,						\
+		   duplicate(VECTOR_ELT(GET_SLOT(y, Matrix_DimNamesSym), \
+					tr ? 0 : 1)));			\
+    SET_SLOT(val, Matrix_DimNamesSym, dn);				\
+    vDims = INTEGER(ALLOC_SLOT(val, Matrix_DimSym, INTSXP, 2));		\
+    vDims[0] = m; vDims[1] = n;						\
+    double *v = REAL(ALLOC_SLOT(val, Matrix_xSym, REALSXP, m * (R_xlen_t) n))
 
-    SET_SLOT(val, Matrix_factorSym, allocVector(VECSXP, 0));
-    SET_SLOT(val, Matrix_DimSym, allocVector(INTSXP, 2));
-    vDims = INTEGER(GET_SLOT(val, Matrix_DimSym));
-    if (xd > 0 && yd > 0 && n > 0 && m > 0) {
-	if (xd != yd)
-	    error(_("Dimensions of x and y are not compatible for %s"),
-		  tr ? "tcrossprod" : "crossprod");
-	vDims[0] = m; vDims[1] = n;
-	SET_SLOT(val, Matrix_xSym, allocVector(REALSXP, m * n));
-	F77_CALL(dgemm)(tr ? "N" : "T", tr ? "T" : "N", &m, &n, &xd, &one,
-			REAL(GET_SLOT(x, Matrix_xSym)), xDims,
-			REAL(GET_SLOT(y, Matrix_xSym)), yDims,
-			&zero, REAL(GET_SLOT(val, Matrix_xSym)), &m);
-	/* establish dimnames */
-	SET_VECTOR_ELT(dn, 0,
-		       duplicate(VECTOR_ELT(GET_SLOT(x, Matrix_DimNamesSym),
-					    tr ? 0 : 1)));
-	SET_VECTOR_ELT(dn, 1,
-		       duplicate(VECTOR_ELT(GET_SLOT(y, Matrix_DimNamesSym),
-					    tr ? 0 : 1)));
-	SET_SLOT(val, Matrix_DimNamesSym, dn);
-    }
+#define DGE_DGE_CROSS_DO(_X_X_, _Y_Y_)					\
+    if (xd > 0 && n > 0 && m > 0)					\
+	F77_CALL(dgemm)(tr ? "N" : "T", tr ? "T" : "N", &m, &n, &xd, &one, \
+			_X_X_, xDims,					\
+			_Y_Y_, yDims, &zero, v, &m FCONE FCONE);	\
+    else								\
+	Memzero(v,  m * (R_xlen_t) n);					\
+    UNPROTECT(2);							\
+    return val
+
+    DGE_DGE_CROSS_1;
+    DGE_DGE_CROSS_DO(REAL(GET_SLOT(x, Matrix_xSym)),
+		     REAL(GET_SLOT(y, Matrix_xSym)));
+}
+
+//!  As  dgeMatrix_dgeMatrix_crossprod(), but x and y can be [dln]geMatrix
+SEXP _geMatrix__geMatrix_crossprod(SEXP x, SEXP y, SEXP trans)
+{
+    DGE_DGE_CROSS_1;
+
+    double *x_x = gematrix_real_x(x, m * (R_xlen_t) xd);
+    double *y_x = gematrix_real_x(y, n * (R_xlen_t) yd);
+
+    DGE_DGE_CROSS_DO(x_x, y_x);
+}
+#undef DGE_DGE_CROSS_1
+#undef DGE_DGE_CROSS_DO
+
+SEXP geMatrix_geMatrix_crossprod(SEXP x, SEXP y, SEXP trans)
+{
+    SEXP gx = PROTECT(dense_as_general(x, '.', 2, 0)),
+	gy = PROTECT(dense_as_general(y, '.', 2, 0)),
+	val = _geMatrix__geMatrix_crossprod(gx, gy, trans);
     UNPROTECT(2);
     return val;
 }
 
 SEXP dgeMatrix_matrix_crossprod(SEXP x, SEXP y, SEXP trans)
 {
-    int tr = asLogical(trans);/* trans=TRUE: tcrossprod(x,y) */
-    SEXP val = PROTECT(NEW_OBJECT(MAKE_CLASS("dgeMatrix"))),
-	dn = PROTECT(allocVector(VECSXP, 2)),
-	yDnms = R_NilValue, yD;
-    int *xDims = INTEGER(GET_SLOT(x, Matrix_DimSym)),
-	*yDims, *vDims, nprot = 2;
-    int m  = xDims[!tr],
-	xd = xDims[ tr];
-    double one = 1.0, zero = 0.0;
-    Rboolean y_has_dimNames;
+#define DGE_MAT_CROSS_1							\
+    int tr = asLogical(trans);/* trans=TRUE: tcrossprod(x,y) */		\
+    SEXP val = PROTECT(NEW_OBJECT_OF_CLASS("dgeMatrix")),		\
+	dn = PROTECT(allocVector(VECSXP, 2)),				\
+	yDnms = R_NilValue, yD;						\
+    int *xDims = INTEGER(GET_SLOT(x, Matrix_DimSym)),			\
+	*yDims, *vDims, nprot = 2;					\
+    int m  = xDims[!tr],						\
+	xd = xDims[ tr];						\
+    double one = 1.0, zero = 0.0;					\
+    Rboolean y_has_dimNames;						\
+									\
+    if (!isReal(y)) {							\
+	if(isInteger(y) || isLogical(y)) {				\
+	    y = PROTECT(coerceVector(y, REALSXP));			\
+	    nprot++;							\
+	}								\
+	else								\
+	    error(_("Argument y must be numeric, integer or logical"));	\
+    }									\
+    if(isMatrix(y)) {							\
+	yDims = INTEGER(getAttrib(y, R_DimSymbol));			\
+	yDnms = getAttrib(y, R_DimNamesSymbol);				\
+	y_has_dimNames = yDnms != R_NilValue;				\
+    } else { /* ! matrix */ 						\
+	yDims = INTEGER(yD = PROTECT(allocVector(INTSXP, 2))); nprot++;	\
+	if(xDims[0] == 1) {						\
+             /* "new" (2014-10-10): "be tolerant" as for R 3.2.0*/ 	\
+	    yDims[0] = 1;						\
+	    yDims[1] = LENGTH(y);					\
+	} else {							\
+	    yDims[0] = LENGTH(y);					\
+	    yDims[1] = 1;						\
+	}								\
+	y_has_dimNames = FALSE;						\
+    }									\
+    int  n = yDims[!tr],/* (m,n) -> result dim */			\
+	yd = yDims[ tr];/* (xd,yd): the conformable dims */		\
+    if (xd != yd)							\
+	error(_("Dimensions of x and y are not compatible for %s"),	\
+	      tr ? "tcrossprod" : "crossprod");				\
+    SET_SLOT(val, Matrix_factorSym, allocVector(VECSXP, 0));		\
+    vDims = INTEGER(ALLOC_SLOT(val, Matrix_DimSym, INTSXP, 2));		\
+    vDims[0] = m; vDims[1] = n;						\
+    /* establish dimnames */						\
+    SET_VECTOR_ELT(dn, 0,						\
+		   duplicate(VECTOR_ELT(GET_SLOT(x, Matrix_DimNamesSym), \
+					tr ? 0 : 1)));			\
+    if(y_has_dimNames)							\
+	SET_VECTOR_ELT(dn, 1,						\
+		       duplicate(VECTOR_ELT(yDnms, tr ? 0 : 1)));	\
+    SET_SLOT(val, Matrix_DimNamesSym, dn);				\
+									\
+    double *v = REAL(ALLOC_SLOT(val, Matrix_xSym, REALSXP,  m * (R_xlen_t) n))
 
-    if (isInteger(y)) {
-	y = PROTECT(coerceVector(y, REALSXP));
-	nprot++;
-    }
-    if (!isReal(y))
-	error(_("Argument y must be numeric or integer"));
-    if(isMatrix(y)) {
-	yDims = INTEGER(getAttrib(y, R_DimSymbol));
-	yDnms = getAttrib(y, R_DimNamesSymbol);
-	y_has_dimNames = yDnms != R_NilValue;
-    } else { // ! matrix
-	yDims = INTEGER(yD = PROTECT(allocVector(INTSXP, 2))); nprot++;
-	yDims[0] = LENGTH(y);
-	yDims[1] = 1;
-	y_has_dimNames = FALSE;
-    }
-    int  n = yDims[!tr],/* (m,n) -> result dim */
-	yd = yDims[ tr];/* (xd,yd): the conformable dims */
-    SET_SLOT(val, Matrix_factorSym, allocVector(VECSXP, 0));
-    SET_SLOT(val, Matrix_DimSym, allocVector(INTSXP, 2));
-    vDims = INTEGER(GET_SLOT(val, Matrix_DimSym));
-    if (xd > 0 && yd > 0 && n > 0 && m > 0) {
-	if (xd != yd)
-	    error(_("Dimensions of x and y are not compatible for %s"),
-		  tr ? "tcrossprod" : "crossprod");
-	vDims[0] = m; vDims[1] = n;
-	SET_SLOT(val, Matrix_xSym, allocVector(REALSXP, m * n));
-	F77_CALL(dgemm)(tr ? "N" : "T", tr ? "T" : "N", &m, &n, &xd, &one,
-			REAL(GET_SLOT(x, Matrix_xSym)), xDims,
-			REAL(y), yDims,
-			&zero, REAL(GET_SLOT(val, Matrix_xSym)), &m);
-	/* establish dimnames */
-	SET_VECTOR_ELT(dn, 0,
-		       duplicate(VECTOR_ELT(GET_SLOT(x, Matrix_DimNamesSym),
-					    tr ? 0 : 1)));
-	if(y_has_dimNames)
-	    SET_VECTOR_ELT(dn, 1,
-			   duplicate(VECTOR_ELT(yDnms, tr ? 0 : 1)));
-	SET_SLOT(val, Matrix_DimNamesSym, dn);
-    }
-    UNPROTECT(nprot);
+#define DGE_MAT_CROSS_DO(_X_X_)						\
+    if (xd > 0 && n > 0 && m > 0)					\
+	F77_CALL(dgemm)(tr ? "N" : "T", tr ? "T" : "N", &m, &n, &xd, &one, \
+			_X_X_, xDims, REAL(y), yDims, 			\
+			&zero, v, &m FCONE FCONE);			\
+    else								\
+	Memzero(v,  m * (R_xlen_t) n);					\
+    UNPROTECT(nprot);							\
+    return val
+
+    DGE_MAT_CROSS_1;
+    DGE_MAT_CROSS_DO(REAL(GET_SLOT(x, Matrix_xSym)));
+}
+
+//! as dgeMatrix_matrix_crossprod() but x can be  [dln]geMatrix
+SEXP _geMatrix_matrix_crossprod(SEXP x, SEXP y, SEXP trans) {
+    DGE_MAT_CROSS_1;
+
+    double *x_x = gematrix_real_x(x, m * (R_xlen_t) xd);
+
+    DGE_MAT_CROSS_DO(x_x);
+}
+
+SEXP geMatrix_matrix_crossprod(SEXP x, SEXP y, SEXP trans) {
+    SEXP dx = PROTECT(dense_as_general(x, '.', 2, 0)),
+	val = _geMatrix_matrix_crossprod(dx, y, trans);
+    UNPROTECT(1);
     return val;
 }
 
+//  right = TRUE:  %*%  is called as  *(y, x, right=TRUE)
+SEXP dgeMatrix_matrix_mm(SEXP a, SEXP bP, SEXP right)
+{
+#define DGE_MAT_MM_1(N_PROT)						\
+    SEXP val= PROTECT(NEW_OBJECT_OF_CLASS("dgeMatrix")),		\
+	 dn = PROTECT(allocVector(VECSXP, 2));				\
+    int nprot = N_PROT + 2,						\
+	*adims = INTEGER(GET_SLOT(a, Matrix_DimSym)),			\
+	*bdims = INTEGER(GET_SLOT(b, Matrix_DimSym)),			\
+	*cdims = INTEGER(ALLOC_SLOT(val, Matrix_DimSym, INTSXP, 2)),	\
+	Rt = asLogical(right), m, k, n;					\
+    double one = 1., zero = 0.;						\
+									\
+    if (Rt) { /* b %*% a : (m x k) (k x n) -> (m x n) */		\
+	m = bdims[0]; k = bdims[1]; n = adims[1];			\
+	if (adims[0] != k)						\
+	    error(_("Matrices are not conformable for multiplication")); \
+    } else {  /* a %*% b : (m x k) (k x n) -> (m x n) */		\
+	m = adims[0]; k = adims[1]; n = bdims[1];			\
+	if (bdims[0] != k)						\
+	    error(_("Matrices are not conformable for multiplication")); \
+    }									\
+									\
+    cdims[0] = m; cdims[1] = n;						\
+    /* establish dimnames */						\
+    SET_VECTOR_ELT(dn, 0, duplicate(					\
+		       VECTOR_ELT(GET_SLOT(Rt ? b : a,			\
+					   Matrix_DimNamesSym), 0)));	\
+    SET_VECTOR_ELT(dn, 1,						\
+		   duplicate(						\
+		       VECTOR_ELT(GET_SLOT(Rt ? a : b,			\
+					   Matrix_DimNamesSym), 1)));	\
+    SET_SLOT(val, Matrix_DimNamesSym, dn);				\
+    double *v = REAL(ALLOC_SLOT(val, Matrix_xSym, REALSXP, m * (R_xlen_t) n))
 
+#define DGE_MAT_MM_DO(_A_X_, _B_X_)					\
+    if (m < 1 || n < 1 || k < 1) {/* zero extent matrices should work */ \
+	Memzero(v, m * (R_xlen_t) n);					\
+    } else {								\
+	if (Rt) { /* b %*% a  */					\
+	    F77_CALL(dgemm) ("N", "N", &m, &n, &k, &one,		\
+			     _B_X_, &m, _A_X_, &k, &zero, v, &m FCONE FCONE); \
+	} else {  /* a %*% b  */					\
+	    F77_CALL(dgemm) ("N", "N", &m, &n, &k, &one,		\
+			     _A_X_, &m,	_B_X_, &k, &zero, v, &m FCONE FCONE); \
+	}								\
+    }									\
+    UNPROTECT(nprot);							\
+    return val
+
+    SEXP b = PROTECT(dense_as_general(bP, 'd', 2, 0));
+    DGE_MAT_MM_1(1);
+    DGE_MAT_MM_DO(REAL(GET_SLOT(a, Matrix_xSym)),
+                  REAL(GET_SLOT(b, Matrix_xSym)));
+}
+
+//! as dgeMatrix_matrix_mm() but a can be  [dln]geMatrix
+SEXP _geMatrix_matrix_mm(SEXP a, SEXP b, SEXP right) {
+    DGE_MAT_MM_1(0);
+    double *a_x = gematrix_real_x(a, k * (R_xlen_t)(Rt ? n : m));
+    double *b_x = gematrix_real_x(b, k * (R_xlen_t)(Rt ? m : n));
+    DGE_MAT_MM_DO(a_x, b_x);
+}
+
+//! %*% -- generalized from dge to *ge():
+SEXP geMatrix_matrix_mm(SEXP a, SEXP b, SEXP right) {
+    SEXP
+	da = PROTECT(dense_as_general(a, '.', 2, 0)),
+	db = PROTECT(dense_as_general(b, '.', 2, 0)),
+	val = _geMatrix_matrix_mm(da, db, right);
+    UNPROTECT(2);
+    return val;
+}
+
+/* MJ: no longer needed ... prefer more general unpackedMatrix_diag_[gs]et() */
+#if 0
 SEXP dgeMatrix_getDiag(SEXP x)
 {
 #define geMatrix_getDiag_1					\
     int *dims = INTEGER(GET_SLOT(x, Matrix_DimSym));		\
     int i, m = dims[0], nret = (m < dims[1]) ? m : dims[1];	\
+    R_xlen_t m1 = m + 1;					\
     SEXP x_x = GET_SLOT(x, Matrix_xSym)
 
     geMatrix_getDiag_1;
@@ -216,7 +363,7 @@ SEXP dgeMatrix_getDiag(SEXP x)
 
 #define geMatrix_getDiag_2			\
     for (i = 0; i < nret; i++) {		\
-	rv[i] = xv[i * (m + 1)];		\
+	rv[i] = xv[i * m1];			\
     }						\
     UNPROTECT(1);				\
     return ret
@@ -238,7 +385,6 @@ SEXP lgeMatrix_getDiag(SEXP x)
 #undef geMatrix_getDiag_1
 #undef geMatrix_getDiag_2
 
-
 SEXP dgeMatrix_setDiag(SEXP x, SEXP d)
 {
 #define geMatrix_setDiag_1					\
@@ -248,16 +394,17 @@ SEXP dgeMatrix_setDiag(SEXP x, SEXP d)
     SEXP r_x = GET_SLOT(ret, Matrix_xSym);			\
     int l_d = LENGTH(d); Rboolean d_full = (l_d == nret);	\
     if (!d_full && l_d != 1)					\
-	error("replacement diagonal has wrong length")
+	error(_("replacement diagonal has wrong length"))
 
     geMatrix_setDiag_1;
     double *dv = REAL(d), *rv = REAL(r_x);
 
 #define geMatrix_setDiag_2			\
+    R_xlen_t m1 = m + 1;                        \
     if(d_full) for (int i = 0; i < nret; i++)	\
-	rv[i * (m + 1)] = dv[i];		\
+	rv[i * m1] = dv[i];			\
     else for (int i = 0; i < nret; i++)		\
-	rv[i * (m + 1)] = *dv;			\
+	rv[i * m1] = *dv;			\
     UNPROTECT(1);				\
     return ret
 
@@ -275,28 +422,30 @@ SEXP lgeMatrix_setDiag(SEXP x, SEXP d)
 #undef geMatrix_setDiag_1
 #undef geMatrix_setDiag_2
 
+/* was unused, not replaced: */
 SEXP dgeMatrix_addDiag(SEXP x, SEXP d)
 {
     int *dims = INTEGER(GET_SLOT(x, Matrix_DimSym)),
 	m = dims[0], nret = (m < dims[1]) ? m : dims[1];
+    R_xlen_t m1 = m + 1;
     SEXP ret = PROTECT(duplicate(x)),
 	r_x = GET_SLOT(ret, Matrix_xSym);
     double *dv = REAL(d), *rv = REAL(r_x);
     int l_d = LENGTH(d); Rboolean d_full = (l_d == nret);
     if (!d_full && l_d != 1)
-	error("diagonal to be added has wrong length");
+	error(_("diagonal to be added has wrong length"));
 
-    if(d_full) for (int i = 0; i < nret; i++) rv[i * (m + 1)] += dv[i];
-    else for (int i = 0; i < nret; i++)	      rv[i * (m + 1)] += *dv;
+    if(d_full) for (int i = 0; i < nret; i++) rv[i * m1] += dv[i];
+    else for (int i = 0; i < nret; i++)	      rv[i * m1] += *dv;
     UNPROTECT(1);
     return ret;
 }
-
+#endif /* MJ */
 
 
 SEXP dgeMatrix_LU_(SEXP x, Rboolean warn_sing)
 {
-    SEXP val = get_factors(x, "LU");
+    SEXP val = get_factor(x, "LU");
     int *dims, npiv, info;
 
     if (val != R_NilValue) /* nothing to do if it's there in 'factors' slot */
@@ -305,9 +454,10 @@ SEXP dgeMatrix_LU_(SEXP x, Rboolean warn_sing)
     if (dims[0] < 1 || dims[1] < 1)
 	error(_("Cannot factor a matrix with zero extents"));
     npiv = (dims[0] < dims[1]) ? dims[0] : dims[1];
-    val = PROTECT(NEW_OBJECT(MAKE_CLASS("denseLU")));
+    val = PROTECT(NEW_OBJECT_OF_CLASS("denseLU"));
     slot_dup(val, x, Matrix_xSym);
     slot_dup(val, x, Matrix_DimSym);
+    slot_dup(val, x, Matrix_DimNamesSym);
     F77_CALL(dgetrf)(dims, dims + 1, REAL(GET_SLOT(val, Matrix_xSym)),
 		     dims,
 		     INTEGER(ALLOC_SLOT(val, Matrix_permSym, INTSXP, npiv)),
@@ -317,8 +467,9 @@ SEXP dgeMatrix_LU_(SEXP x, Rboolean warn_sing)
     else if (info > 0 && warn_sing)
 	warning(_("Exact singularity detected during LU decomposition: %s, i=%d."),
 		"U[i,i]=0", info);
+    set_factor(x, "LU", val);
     UNPROTECT(1);
-    return set_factors(x, val, "LU");
+    return val;
 }
 // FIXME: also allow an interface to LAPACK's  dgesvx()  which uses LU fact.
 //        and then optionally does "equilibration" (row and column scaling)
@@ -369,7 +520,7 @@ SEXP dgeMatrix_solve(SEXP a)
     double aNorm = get_norm(a, "1");
 
     /* the LU decomposition : */
-    SEXP val = PROTECT(NEW_OBJECT(MAKE_CLASS("dgeMatrix"))),
+    SEXP val = PROTECT(NEW_OBJECT_OF_CLASS("dgeMatrix")),
 	lu = dgeMatrix_LU_(a, TRUE);
     int *dims = INTEGER(GET_SLOT(lu, Matrix_DimSym)),
 	*pivot = INTEGER(GET_SLOT(lu, Matrix_permSym));
@@ -390,10 +541,10 @@ SEXP dgeMatrix_solve(SEXP a)
         double rcond;
         F77_CALL(dgecon)("1", dims, x, dims, &aNorm, &rcond,
                          (double *) R_alloc(4*dims[0], sizeof(double)),
-                         (int *) R_alloc(dims[0], sizeof(int)), &info);
+                         (int *) R_alloc(dims[0], sizeof(int)), &info FCONE);
         if (info)
             error(_("error [%d] from Lapack 'dgecon()'"), info);
-        if(rcond < DOUBLE_EPS)
+        if(rcond < DBL_EPSILON)
             error(_("Lapack dgecon(): system computationally singular, reciprocal condition number = %g"),
 		  rcond);
 
@@ -412,85 +563,25 @@ SEXP dgeMatrix_solve(SEXP a)
 
 SEXP dgeMatrix_matrix_solve(SEXP a, SEXP b)
 {
-    SEXP val = PROTECT(dup_mMatrix_as_dgeMatrix(b)),
+    SEXP val = PROTECT(dense_as_general(b, 'd', 2, 0)),
 	lu = PROTECT(dgeMatrix_LU_(a, TRUE));
     int *adims = INTEGER(GET_SLOT(lu, Matrix_DimSym)),
 	*bdims = INTEGER(GET_SLOT(val, Matrix_DimSym));
     int info, n = bdims[0], nrhs = bdims[1];
 
-    if (*adims != *bdims || bdims[1] < 1 || *adims < 1 || *adims != adims[1])
+    if (adims[0] != n || adims[1] != n)
 	error(_("Dimensions of system to be solved are inconsistent"));
-    F77_CALL(dgetrs)("N", &n, &nrhs, REAL(GET_SLOT(lu, Matrix_xSym)), &n,
-		     INTEGER(GET_SLOT(lu, Matrix_permSym)),
-		     REAL(GET_SLOT(val, Matrix_xSym)), &n, &info);
-    if (info)
-	error(_("Lapack routine dgetrs: system is exactly singular"));
+    if(nrhs >= 1 && n >= 1) {
+	F77_CALL(dgetrs)("N", &n, &nrhs, REAL(GET_SLOT(lu, Matrix_xSym)), &n,
+			 INTEGER(GET_SLOT(lu, Matrix_permSym)),
+			 REAL(GET_SLOT(val, Matrix_xSym)), &n, &info FCONE);
+	if (info)
+	    error(_("Lapack routine dgetrs: system is exactly singular"));
+    }
     UNPROTECT(2);
     return val;
 }
 
-//  right = TRUE:  %*%  is called as  *(y, x, right=TRUE)
-SEXP dgeMatrix_matrix_mm(SEXP a, SEXP bP, SEXP right)
-{
-    SEXP b = PROTECT(mMatrix_as_dgeMatrix(bP)),
-	val= PROTECT(NEW_OBJECT(MAKE_CLASS("dgeMatrix"))),
-	dn = PROTECT(allocVector(VECSXP, 2));
-    int *adims = INTEGER(GET_SLOT(a, Matrix_DimSym)),
-	*bdims = INTEGER(GET_SLOT(b, Matrix_DimSym)),
-	*cdims = INTEGER(ALLOC_SLOT(val, Matrix_DimSym, INTSXP, 2));
-    double one = 1., zero = 0.;
-
-    if (asLogical(right)) { // b %*% a
-	int m = bdims[0], n = adims[1], k = bdims[1];
-	if (adims[0] != k)
-	    error(_("Matrices are not conformable for multiplication"));
-	cdims[0] = m; cdims[1] = n;
-	if (m < 1 || n < 1 || k < 1) {
-/* 	    error(_("Matrices with zero extents cannot be multiplied")); */
-	    ALLOC_SLOT(val, Matrix_xSym, REALSXP, m * n);
-	} else {
-	    F77_CALL(dgemm) ("N", "N", &m, &n, &k, &one,
-			     REAL(GET_SLOT(b, Matrix_xSym)), &m,
-			     REAL(GET_SLOT(a, Matrix_xSym)), &k, &zero,
-			     REAL(ALLOC_SLOT(val, Matrix_xSym, REALSXP, m * n)),
-			     &m);
-	    SET_VECTOR_ELT(dn, 0,
-			   duplicate(VECTOR_ELT(GET_SLOT(b, Matrix_DimNamesSym),
-						0)));
-	    SET_VECTOR_ELT(dn, 1,
-			   duplicate(VECTOR_ELT(GET_SLOT(a, Matrix_DimNamesSym),
-						1)));
-	}
-    } else {  // a %*% b
-	int m = adims[0], n = bdims[1], k = adims[1];
-
-	if (bdims[0] != k)
-	    error(_("Matrices are not conformable for multiplication"));
-	cdims[0] = m; cdims[1] = n;
-	if (m < 1 || n < 1 || k < 1) {
-/* 	    error(_("Matrices with zero extents cannot be multiplied")); */
-	    ALLOC_SLOT(val, Matrix_xSym, REALSXP, m * n);
-	} else {
-	    F77_CALL(dgemm) ("N", "N", &m, &n, &k, &one,
-			     REAL(GET_SLOT(a, Matrix_xSym)), &m,
-			     REAL(GET_SLOT(b, Matrix_xSym)), &k, &zero,
-			     REAL(ALLOC_SLOT(val, Matrix_xSym, REALSXP, m * n)),
-			     &m);
-	    SET_VECTOR_ELT(dn, 0,
-			   duplicate(VECTOR_ELT(GET_SLOT(a, Matrix_DimNamesSym),
-						0)));
-	    SET_VECTOR_ELT(dn, 1,
-			   duplicate(VECTOR_ELT(GET_SLOT(b, Matrix_DimNamesSym),
-						1)));
-	}
-    }
-    /* establish dimnames */
-    SET_SLOT(val, Matrix_DimNamesSym, dn);
-    UNPROTECT(3);
-    return val;
-}
-
-
 SEXP dgeMatrix_svd(SEXP x, SEXP nnu, SEXP nnv)
 {
     int /* nu = asInteger(nnu),
@@ -503,8 +594,10 @@ SEXP dgeMatrix_svd(SEXP x, SEXP nnu, SEXP nnv)
 	int m = dims[0], n = dims[1], mm = (m < n)?m:n,
 	    lwork = -1, info;
 	double tmp, *work;
-	int *iwork = Alloca(8 * mm, int);
-	R_CheckStack();
+	int *iwork, n_iw = 8 * mm;
+	if(8 * (double)mm != n_iw) // integer overflow
+	    error(_("dgeMatrix_svd(x,*): dim(x)[j] = %d is too large"), mm);
+	Calloc_or_Alloca_TO(iwork, n_iw, int);
 
 	SET_VECTOR_ELT(val, 0, allocVector(REALSXP, mm));
 	SET_VECTOR_ELT(val, 1, allocMatrix(REALSXP, m, mm));
@@ -513,16 +606,18 @@ SEXP dgeMatrix_svd(SEXP x, SEXP nnu, SEXP nnv)
 			 REAL(VECTOR_ELT(val, 0)),
 			 REAL(VECTOR_ELT(val, 1)), &m,
 			 REAL(VECTOR_ELT(val, 2)), &mm,
-			 &tmp, &lwork, iwork, &info);
+			 &tmp, &lwork, iwork, &info FCONE);
 	lwork = (int) tmp;
-	work = Alloca(lwork, double);
-	R_CheckStack();
+	Calloc_or_Alloca_TO(work, lwork, double);
+
 	F77_CALL(dgesdd)("S", &m, &n, xx, &m,
 			 REAL(VECTOR_ELT(val, 0)),
 			 REAL(VECTOR_ELT(val, 1)), &m,
 			 REAL(VECTOR_ELT(val, 2)), &mm,
-			 work, &lwork, iwork, &info);
+			 work, &lwork, iwork, &info FCONE);
 
+	Free_FROM(iwork, n_iw);
+	Free_FROM(work, lwork);
     }
     UNPROTECT(1);
     return val;
@@ -552,17 +647,18 @@ SEXP dgeMatrix_exp(SEXP x)
     const double one = 1.0, zero = 0.0;
     const int i1 = 1;
     int *Dims = INTEGER(GET_SLOT(x, Matrix_DimSym));
-    const int n = Dims[1], nsqr = n * n, np1 = n + 1;
+    const int n = Dims[1];
+    const R_xlen_t n_ = n, np1 = n + 1, nsqr = n_ * n_; // nsqr = n^2
 
     SEXP val = PROTECT(duplicate(x));
     int i, ilo, ilos, ihi, ihis, j, sqpow;
-    int *pivot = Calloc(n, int);
-    double *dpp = Calloc(nsqr, double), /* denominator power Pade' */
-	*npp = Calloc(nsqr, double), /* numerator power Pade' */
-	*perm = Calloc(n, double),
-	*scale = Calloc(n, double),
+    int *pivot = R_Calloc(n, int);
+    double *dpp = R_Calloc(nsqr, double), /* denominator power Pade' */
+	*npp = R_Calloc(nsqr, double), /* numerator power Pade' */
+	*perm = R_Calloc(n, double),
+	*scale = R_Calloc(n, double),
 	*v = REAL(GET_SLOT(val, Matrix_xSym)),
-	*work = Calloc(nsqr, double), inf_norm, m1_j/*= (-1)^j */, trshift;
+	*work = R_Calloc(nsqr, double), inf_norm, m1_j/*= (-1)^j */, trshift;
     R_CheckStack();
 
     if (n < 1 || Dims[0] != n)
@@ -582,40 +678,40 @@ SEXP dgeMatrix_exp(SEXP x)
     }
 
     /* Preconditioning 2. Balancing with dgebal. */
-    F77_CALL(dgebal)("P", &n, v, &n, &ilo, &ihi, perm, &j);
+    F77_CALL(dgebal)("P", &n, v, &n, &ilo, &ihi, perm, &j FCONE);
     if (j) error(_("dgeMatrix_exp: LAPACK routine dgebal returned %d"), j);
-    F77_CALL(dgebal)("S", &n, v, &n, &ilos, &ihis, scale, &j);
+    F77_CALL(dgebal)("S", &n, v, &n, &ilos, &ihis, scale, &j FCONE);
     if (j) error(_("dgeMatrix_exp: LAPACK routine dgebal returned %d"), j);
 
     /* Preconditioning 3. Scaling according to infinity norm */
-    inf_norm = F77_CALL(dlange)("I", &n, &n, v, &n, work);
+    inf_norm = F77_CALL(dlange)("I", &n, &n, v, &n, work FCONE);
     sqpow = (inf_norm > 0) ? (int) (1 + log(inf_norm)/log(2.)) : 0;
     if (sqpow < 0) sqpow = 0;
     if (sqpow > 0) {
 	double scale_factor = 1.0;
 	for (i = 0; i < sqpow; i++) scale_factor *= 2.;
-	for (i = 0; i < nsqr; i++) v[i] /= scale_factor;
+	for (R_xlen_t i = 0; i < nsqr; i++) v[i] /= scale_factor;
     }
 
     /* Pade' approximation. Powers v^8, v^7, ..., v^1 */
-    AZERO(npp, nsqr);
-    AZERO(dpp, nsqr);
+    AZERO(npp, nsqr, 0.0);
+    AZERO(dpp, nsqr, 0.0);
     m1_j = -1;
     for (j = 7; j >=0; j--) {
 	double mult = padec[j];
 	/* npp = m * npp + padec[j] *m */
 	F77_CALL(dgemm)("N", "N", &n, &n, &n, &one, v, &n, npp, &n,
-			&zero, work, &n);
-	for (i = 0; i < nsqr; i++) npp[i] = work[i] + mult * v[i];
+			&zero, work, &n FCONE FCONE);
+	for (R_xlen_t i = 0; i < nsqr; i++) npp[i] = work[i] + mult * v[i];
 	/* dpp = m * dpp + (m1_j * padec[j]) * m */
 	mult *= m1_j;
 	F77_CALL(dgemm)("N", "N", &n, &n, &n, &one, v, &n, dpp, &n,
-			&zero, work, &n);
-	for (i = 0; i < nsqr; i++) dpp[i] = work[i] + mult * v[i];
+			&zero, work, &n FCONE FCONE);
+	for (R_xlen_t i = 0; i < nsqr; i++) dpp[i] = work[i] + mult * v[i];
 	m1_j *= -1;
     }
     /* Zero power */
-    for (i = 0; i < nsqr; i++) dpp[i] *= -1.;
+    for (R_xlen_t i = 0; i < nsqr; i++) dpp[i] *= -1.;
     for (j = 0; j < n; j++) {
 	npp[j * np1] += 1.;
 	dpp[j * np1] += 1.;
@@ -624,7 +720,7 @@ SEXP dgeMatrix_exp(SEXP x)
     /* Pade' approximation is solve(dpp, npp) */
     F77_CALL(dgetrf)(&n, &n, dpp, &n, pivot, &j);
     if (j) error(_("dgeMatrix_exp: dgetrf returned error code %d"), j);
-    F77_CALL(dgetrs)("N", &n, &n, dpp, &n, pivot, npp, &n, &j);
+    F77_CALL(dgetrs)("N", &n, &n, dpp, &n, pivot, npp, &n, &j FCONE);
     if (j) error(_("dgeMatrix_exp: dgetrs returned error code %d"), j);
     Memcpy(v, npp, nsqr);
 
@@ -632,13 +728,15 @@ SEXP dgeMatrix_exp(SEXP x)
     /* Preconditioning 3: square the result for every power of 2 */
     while (sqpow--) {
 	F77_CALL(dgemm)("N", "N", &n, &n, &n, &one, v, &n, v, &n,
-			&zero, work, &n);
+			&zero, work, &n FCONE FCONE);
 	Memcpy(v, work, nsqr);
     }
     /* Preconditioning 2: apply inverse scaling */
-    for (j = 0; j < n; j++)
+    for (j = 0; j < n; j++) {
+	R_xlen_t jn = j * n_;
 	for (i = 0; i < n; i++)
-	    v[i + j * n] *= scale[i]/scale[j];
+	    v[i + jn] *= scale[i]/scale[j];
+    }
 
 
     /* 2 b) Inverse permutation  (if not the identity permutation) */
@@ -647,7 +745,7 @@ SEXP dgeMatrix_exp(SEXP x)
 
 #define SWAP_ROW(I,J) F77_CALL(dswap)(&n, &v[(I)], &n, &v[(J)], &n)
 
-#define SWAP_COL(I,J) F77_CALL(dswap)(&n, &v[(I)*n], &i1, &v[(J)*n], &i1)
+#define SWAP_COL(I,J) F77_CALL(dswap)(&n, &v[(I)*n_], &i1, &v[(J)*n_], &i1)
 
 #define RE_PERMUTE(I)				\
 	int p_I = (int) (perm[I]) - 1;		\
@@ -668,11 +766,11 @@ SEXP dgeMatrix_exp(SEXP x)
     /* Preconditioning 1: Trace normalization */
     if (trshift > 0.) {
 	double mult = exp(trshift);
-	for (i = 0; i < nsqr; i++) v[i] *= mult;
+	for (R_xlen_t i = 0; i < nsqr; i++) v[i] *= mult;
     }
 
     /* Clean up */
-    Free(work); Free(scale); Free(perm); Free(npp); Free(dpp); Free(pivot);
+    R_Free(work); R_Free(scale); R_Free(perm); R_Free(npp); R_Free(dpp); R_Free(pivot);
     UNPROTECT(1);
     return val;
 }
@@ -699,71 +797,80 @@ SEXP dgeMatrix_Schur(SEXP x, SEXP vectors, SEXP isDGE)
     n = dims[0];
     if (n != dims[1] || n < 1)
 	error(_("dgeMatrix_Schur: argument x must be a non-null square matrix"));
+    const R_xlen_t n2 = ((R_xlen_t)n) * n; // = n^2
+
     SET_VECTOR_ELT(val, 0, allocVector(REALSXP, n));
     SET_VECTOR_ELT(val, 1, allocVector(REALSXP, n));
     SET_VECTOR_ELT(val, 2, allocMatrix(REALSXP, n, n));
     Memcpy(REAL(VECTOR_ELT(val, 2)),
 	   REAL(is_dge ? GET_SLOT(x, Matrix_xSym) : x),
-	   n * n);
+	   n2);
     SET_VECTOR_ELT(val, 3, allocMatrix(REALSXP, vecs ? n : 0, vecs ? n : 0));
     F77_CALL(dgees)(vecs ? "V" : "N", "N", NULL, dims, (double *) NULL, dims, &izero,
 		    (double *) NULL, (double *) NULL, (double *) NULL, dims,
-		    &tmp, &lwork, (int *) NULL, &info);
+		    &tmp, &lwork, (int *) NULL, &info FCONE FCONE);
     if (info) error(_("dgeMatrix_Schur: first call to dgees failed"));
     lwork = (int) tmp;
-    work = Alloca(lwork, double);
-    R_CheckStack();
+    Calloc_or_Alloca_TO(work, lwork, double);
+
     F77_CALL(dgees)(vecs ? "V" : "N", "N", NULL, dims, REAL(VECTOR_ELT(val, 2)), dims,
 		    &izero, REAL(VECTOR_ELT(val, 0)), REAL(VECTOR_ELT(val, 1)),
 		    REAL(VECTOR_ELT(val, 3)), dims, work, &lwork,
-		    (int *) NULL, &info);
+		    (int *) NULL, &info FCONE FCONE);
+    Free_FROM(work, lwork);
     if (info) error(_("dgeMatrix_Schur: dgees returned code %d"), info);
     UNPROTECT(nprot);
     return val;
 } // dgeMatrix_Schur
 
+// colSums(), colMeans(), rowSums() and rowMeans() -- called from ../R/colSums.R
 SEXP dgeMatrix_colsums(SEXP x, SEXP naRmP, SEXP cols, SEXP mean)
 {
-    int keepNA = !asLogical(naRmP);
+    int keepNA = !asLogical(naRmP); // <==>  na.rm = FALSE,  the default
     int doMean = asLogical(mean);
     int useCols = asLogical(cols);
     int *dims = INTEGER(GET_SLOT(x, Matrix_DimSym));
     int i, j, m = dims[0], n = dims[1];
+    R_xlen_t m_ = (R_xlen_t) m; // ==> use "long-integer" arithmetic for indices
     SEXP ans = PROTECT(allocVector(REALSXP, (useCols) ? n : m));
     double *aa = REAL(ans), *xx = REAL(GET_SLOT(x, Matrix_xSym));
 
     if (useCols) {  /* col(Sums|Means) : */
-	int cnt = m;
-	for (j = 0; j < n; j++) {
-	    double *rx = xx + m * j;
+	R_xlen_t cnt = m_; // := number of 'valid' entries in current column
+	for (j = 0; j < n; j++) { // column j :
+	    double *x_j = xx + m_ * j, s = 0.;
 
-	    aa[j] = 0;
 	    if (keepNA)
-		for (i = 0; i < m; i++) aa[j] += rx[i];
+		for (i = 0; i < m; i++) s += x_j[i];
 	    else {
 		cnt = 0;
 		for (i = 0; i < m; i++)
-		    if (!ISNAN(rx[i])) {cnt++; aa[j] += rx[i];}
+		    if (!ISNAN(x_j[i])) {cnt++; s += x_j[i];}
 	    }
 	    if (doMean) {
-		if (cnt > 0) aa[j] /= cnt; else aa[j] = NA_REAL;
+		if (cnt > 0) s /= cnt; else s = NA_REAL;
 	    }
+	    aa[j] = s;
 	}
     } else { /* row(Sums|Means) : */
-	double *Count = ((!keepNA) && doMean) ?
-	    Alloca(m, double) : (double*)NULL ;
-	R_CheckStack();
+	Rboolean do_count = (!keepNA) && doMean;
+	int *cnt = (int*) NULL;
+	if (do_count) Calloc_or_Alloca_TO(cnt, m, int);
 
-	for (i = 0; i < m; i++) aa[i] = 0.0;
+	// (taking care to access x contiguously: vary i inside j)
+	for (i = 0; i < m; i++) {
+	    aa[i] = 0.;
+	    if(do_count) cnt[i] = 0;
+	}
 	for (j = 0; j < n; j++) {
 	    if (keepNA)
-		for (i = 0; i < m; i++) aa[i] += xx[i + j * m];
+		for (i = 0; i < m; i++) aa[i] += xx[i + j * m_];
 	    else
 		for (i = 0; i < m; i++) {
-		    double el = xx[i + j * m];
+		    double el = xx[i + j * m_];
 		    if (!ISNAN(el)) {
 			aa[i] += el;
-			if (doMean) Count[i]++;
+			if (doMean) cnt[i]++;
 		    }
 		}
 	}
@@ -772,10 +879,14 @@ SEXP dgeMatrix_colsums(SEXP x, SEXP naRmP, SEXP cols, SEXP mean)
 		for (i = 0; i < m; i++) aa[i] /= n;
 	    else
 		for (i = 0; i < m; i++)
-		    aa[i] = (Count[i]>0)? aa[i]/Count[i]: NA_REAL;
+		    aa[i] = (cnt[i] > 0) ? aa[i]/cnt[i] : NA_REAL;
 	}
+	if (do_count) Free_FROM(cnt, m);
     }
 
+    SEXP nms = VECTOR_ELT(GET_SLOT(x, Matrix_DimNamesSym), useCols ? 1 : 0);
+    if(!isNull(nms))
+	setAttrib(ans, R_NamesSymbol, duplicate(nms));
     UNPROTECT(1);
     return ans;
 }
