@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1999--2004  The R Development Core Team
+ *  Copyright (C) 1999-2022  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,8 +13,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  https://www.R-project.org/Licenses/
  */
 
 
@@ -27,10 +27,10 @@
 #define USE_MDI 1
 #endif
 
-#include "Defn.h"
+#include <Defn.h>
+#include <Fileio.h>
 #include <stdio.h>
 #include <Startup.h>
-extern UImode  CharacterMode;
 #include "graphapp/ga.h"
 #include "graphapp/graphapp.h"
 #include "graphapp/stdimg.h"
@@ -41,30 +41,36 @@ extern UImode  CharacterMode;
 #include "rui.h"
 #include "editor.h"
 
+/* from sysutils.c */
+void reEnc2(const char *x, char *y, int ny,
+	    cetype_t ce_in, cetype_t ce_out, int subst);
+
 #define gettext GA_gettext
 
 #define MCHECK(a) if (!(a)) {del(c); return NULL;}
-RECT *RgetMDIsize();
+RECT *RgetMDIsize(void);
 
 /* Pointers to currently open editors */
 static editor REditors[MAXNEDITORS];
 static int neditors  = 0;
 static Rboolean fix_editor_up = FALSE;
 
-static EditorData neweditordata (int file, char *filename)
+static EditorData neweditordata (int file, const char *filename)
 {
     EditorData p;
     p = (EditorData) malloc(sizeof(struct structEditorData));
     p->file = file;
-    p->filename = (char *) malloc(_MAX_PATH*sizeof(char));
+    /* need space for terminator, and to copy it */
+    p->filename = (char *) malloc((MAX_PATH+1)*sizeof(char));
     if (filename)
-	strncpy(p->filename, filename, _MAX_PATH);
+	strncpy(p->filename, filename, MAX_PATH+1);
     p->title = (char *) malloc((EDITORMAXTITLE + 1)*sizeof(char));
     p->title[EDITORMAXTITLE] = p->title[0] = '\0';
     return p;
 }
 
-void deleditordata(EditorData p){
+void deleditordata(EditorData p)
+{
     if (p->stealconsole)
 	fix_editor_up = FALSE;
     free(p->filename);
@@ -74,41 +80,63 @@ void deleditordata(EditorData p){
     free(p);
 }
 
-static void editor_set_title(editor c, char *title) {
+static void editor_set_title(editor c, const char *title)
+{
     char wtitle[EDITORMAXTITLE+1];
     textbox t = getdata(c);
     EditorData p = getdata(t);
     strncpy(wtitle, title, EDITORMAXTITLE);
     wtitle[EDITORMAXTITLE] = '\0';
     strcpy(p->title, wtitle);
-    strncat(wtitle, " - R Editor", EDITORMAXTITLE);
+    if (strlen(wtitle) + strlen(G_("R Editor")) + 3 < EDITORMAXTITLE) {
+	strcat(wtitle, " - ");
+	strcat(wtitle, G_("R Editor"));
+    }
     settext(c, wtitle);
 }
 
 /*** FILE MANAGEMENT FUNCTIONS ***/
 
-static void editor_load_file(editor c, char *name)
+FILE *R_wfopen(const wchar_t *filename, const wchar_t *mode);
+
+static void editor_load_file(editor c, const char *name, int enc)
 {
     textbox t = getdata(c);
     EditorData p = getdata(t);
     FILE *f;
-    char *buffer=NULL, tmp[_MAX_PATH+30];
-    long num=1, bufsize;
-    f = fopen(name, "r");
-    if (f == NULL)
+    char *buffer = NULL, tmp[MAX_PATH+50], tname[MAX_PATH+1];
+    const char *sname;
+    long num = 1, bufsize;
+
+    if(enc == CE_UTF8) {
+	wchar_t wname[MAX_PATH+1];
+	Rf_utf8towcs(wname, name, MAX_PATH+1);
+	f = R_wfopen(wname, L"r");
+	reEnc2(name, tname, MAX_PATH+1, CE_UTF8, CE_NATIVE, 3);
+	sname = tname;
+    } else {
+	f = R_fopen(name, "r");
+	sname = name;
+    }
+    if (f == NULL) {
+	snprintf(tmp, MAX_PATH+50, 
+		 G_("unable to open file %s for reading"), sname);
+	R_ShowMessage(tmp);
 	return;
+    }
     p->file = 1;
-    strncpy(p->filename, name, _MAX_PATH);
+    strncpy(p->filename, name, MAX_PATH+1);
     bufsize = 0;
     while (num > 0) {
-	buffer = realloc(buffer, bufsize+3000+1);
-	num = fread(buffer+bufsize, 1, 3000-1, f);
+	buffer = realloc(buffer, bufsize + 3000 + 1);
+	num = fread(buffer + bufsize, 1, 3000 - 1, f);
 	if (num >= 0) {
 	    bufsize += num;
 	    buffer[bufsize] = '\0';
 	}
 	else {
-	    snprintf(tmp, _MAX_PATH+30, "Couldn't read from file %s", name);
+	    snprintf(tmp, MAX_PATH+50, 
+		     G_("Could not read from file '%s'"), sname);
 	    askok(tmp);
 	}
     }
@@ -119,17 +147,28 @@ static void editor_load_file(editor c, char *name)
     fclose(f);
 }
 
-static void editor_save_file(editor c, char *name)
+static void editor_save_file(editor c, const char *name, int enc)
 {
     textbox t = getdata(c);
     FILE *f;
-    char buf[_MAX_PATH+30];
+    char buf[MAX_PATH+30], tname[MAX_PATH+1];
+    const char *sname;
+
     if (name == NULL)
 	return;
     else {
-	f = fopen(name, "w");
+	if(enc == CE_UTF8) {
+	    wchar_t wname[MAX_PATH+1];
+	    Rf_utf8towcs(wname, name, MAX_PATH+1);
+	    reEnc2(name, tname, MAX_PATH+1, CE_UTF8, CE_NATIVE, 3);
+	    sname = tname;
+	    f = R_wfopen(wname, L"w");
+	} else {
+	    sname = name;
+	    f = R_fopen(sname, "w");
+	}
 	if (f == NULL) {
-	    snprintf(buf, _MAX_PATH+30, "Couldn't save file %s", name);
+	    snprintf(buf, MAX_PATH+30, G_("Could not save file '%s'"), sname);
 	    askok(buf);
 	    return;
 	}
@@ -138,30 +177,33 @@ static void editor_save_file(editor c, char *name)
     }
 }
 
-static void editorsaveas(editor c) {
+static void editorsaveas(editor c)
+{
     textbox t = getdata(c);
     EditorData p = getdata(t);
-    char *current_name = (p->file ? p->filename : "");
-    char *name;
-    setuserfilter("R files (*.R)\0*.R\0S files (*.q, *.ssc, *.S)\0*.q;*.ssc;*.S\0All files (*.*)\0*.*\0\0");
-    name = askfilesave("Save script as", current_name);
-    if (name == NULL)
-	return;
-    else {
-	editor_save_file(c, name);
+    wchar_t *wname;
+
+    setuserfilterW(L"R files (*.R)\0*.R\0S files (*.q, *.ssc, *.S)\0*.q;*.ssc;*.S\0All files (*.*)\0*.*\0\0");
+    wname = askfilesaveW(G_("Save script as"), "");
+    if (wname) {
+	char name[4*MAX_PATH+1];
+	wcstoutf8(name, wname, sizeof(name));
+	/* now check if it has an extension */
+	char *q = strchr(name, '.');
+	if(!q) strncat(name, ".R", 4*MAX_PATH);
+	editor_save_file(c, name, CE_UTF8);
 	p->file = 1;
-	strncpy(p->filename, name, _MAX_PATH);
+	reEnc2(name, p->filename, MAX_PATH+1, CE_UTF8, CE_NATIVE, 3);
 	gsetmodified(t, 0);
-	editor_set_title(c, name);
-	show(c);
+	editor_set_title(c, p->filename);
     }
+    show(c);
 }
 
 static void menueditorsaveas(control m)
 {
     editor c = getdata(m);
     editorsaveas(c);
-    show(c);
 }
 
 static void editorsave(editor c)
@@ -169,8 +211,7 @@ static void editorsave(editor c)
     textbox t = getdata(c);
     EditorData p = getdata(t);
     if (p->file) {  /* save existing file without prompt */
-	char *current_name = p->filename;
-	editor_save_file(c, current_name);
+	editor_save_file(c, p->filename, CE_UTF8);
 	gsetmodified(t, 0);
     }
     /* if new file then prompt for name to save as */
@@ -184,16 +225,16 @@ static void menueditorsave(control m)
     show(c); /* steal focus back from tool button */
 }
 
-/* global console configuration variables */
-char fontname[LF_FACESIZE+1];
-int fontsty, pointsize;
+/* global console configuration variables, from console.c */
+extern char fontname[LF_FACESIZE+4];
+extern int fontsty, pointsize;
 
 static void editorprint(control m)
 {
     printer lpr;
     font f;
     textbox t = getdata(m);
-    char *contents = gettext(t);
+    const char *contents = gettext(t);
     char msg[LF_FACESIZE + 128];
     char *linebuf = NULL;
     int cc, rr, fh, page, linep, i, j, istartline;
@@ -201,7 +242,7 @@ static void editorprint(control m)
 
     if (!(lpr = newprinter(0.0, 0.0, ""))) return;
     f = gnewfont(lpr, strcmp(fontname, "FixedFont") ? fontname : "Courier New",
-		 fontsty, pointsize, 0.0);
+		 fontsty, pointsize, 0.0, 1);
     top = devicepixelsy(lpr) / 5;
     left = devicepixelsx(lpr) / 5;
     fh = fontheight(f);
@@ -214,8 +255,9 @@ static void editorprint(control m)
     while (i < strlen(contents)) {
 	if ( linep + fh >= rr ) { /* new page */
 	    if (page > 1) nextpage(lpr);
-	    sprintf(msg, "Page %d", page++);
-  	    gdrawstr(lpr, f, Black, pt(cc - gstrwidth(lpr, f, msg) - 1, top), msg);
+	    snprintf(msg, LF_FACESIZE + 128, "Page %d", page++);
+	    gdrawstr(lpr, f, Black, pt(cc - gstrwidth(lpr, f, msg) - 1, top),
+		     msg);
 	    linep = top + 2*fh;
 	}
 	j = 0;
@@ -226,7 +268,7 @@ static void editorprint(control m)
 	linebuf = realloc(linebuf, (j+1)*sizeof(char));
 	strncpy(linebuf, &contents[istartline], j);
 	linebuf[j] = '\0';
-  	gdrawstr(lpr, f, Black, pt(left, linep), linebuf);
+	gdrawstr(lpr, f, Black, pt(left, linep), linebuf);
 	linep += fh;
 	++i;
     }
@@ -240,10 +282,12 @@ static void editorconsole(editor c)
     show(RConsole);
 }
 
-/* Remove global pointer to editor when closing an editor. Fill the gap in the array with the last editor in the list */
+/* Remove global pointer to editor when closing an editor. Fill the
+ * gap in the array with the last editor in the list */
 
-static void editorupdateglobals(editor c) {
-    int i=0;
+static void editorupdateglobals(editor c)
+{
+    int i = 0;
     if (neditors > 1) {
 	while (REditors[i] != c) ++i;
 	if (i < neditors - 1)
@@ -254,16 +298,19 @@ static void editorupdateglobals(editor c) {
 
 /* Hooks called when editor window is destroyed */
 
-static void editordel(editor c) {
+static void editordel(editor c)
+{
     editorupdateglobals(c);
 }
 
-static void textboxdel(textbox t) {
+static void textboxdel(textbox t)
+{
     EditorData p = getdata(t);
     deleditordata(p);
 }
 
-int editorchecksave(editor c) {
+int editorchecksave(editor c)
+{
     textbox t = getdata(c);
     EditorData p = getdata(t);
     int save;
@@ -280,8 +327,7 @@ int editorchecksave(editor c) {
 	case NO:
 	    break;
 	case CANCEL:
-	    return 1; /* used in rui.c (closeconsole) to abort closing
-			 the whole of Rgui */
+	    return 1;
 	}
     }
     return 0;
@@ -301,19 +347,21 @@ static void menueditorclose(control m)
 
 /* Called when exiting Rgui, check if any open editors need saving */
 
-void editorcleanall()
+void editorcleanall(void)
 {
     int i;
     for (i = neditors-1;  i >= 0; --i) {
-	if (editorchecksave(REditors[i]))
+	if (editorchecksave(REditors[i])) {
+	    R_ProcessEvents();  // see R_CleanUp
 	    jump_to_toplevel();
+	}
 	del(REditors[i]);
     }
 }
 
-static void editornew()
+static void editornew(void)
 {
-    Rgui_Edit("", "", 0);
+    Rgui_Edit("", CE_NATIVE, "", 0);
 }
 
 void menueditornew(control m)
@@ -321,15 +369,18 @@ void menueditornew(control m)
     editornew();
 }
 
-static void editoropen(char *default_name)
+static void editoropen(const char *default_name)
 {
-    char *name;
+    wchar_t *wname;
+    char name[4*MAX_PATH + 1], title[4*MAX_PATH];
+
     int i; textbox t; EditorData p;
-    setuserfilter("R files (*.R)\0*.R\0S files (*.q, *.ssc, *.S)\0*.q;*.ssc;*.S\0All files (*.*)\0*.*\0\0");
-    name = askfilename("Open script", default_name); /* returns NULL if open dialog cancelled */
-    if (name) {
+    setuserfilterW(L"R files (*.R)\0*.R\0S files (*.q, *.ssc, *.S)\0*.q;*.ssc;*.S\0All files (*.*)\0*.*\0\0");
+    wname = askfilenameW(G_("Open script"), default_name); /* returns NULL if open dialog cancelled */
+    if (wname) {
+	wcstoutf8(name, wname, sizeof(name));
 	/* check if file is already open in an editor. If so, close and open again */
-	for (i=0; i<neditors; ++i) {
+	for (i = 0; i < neditors; ++i) {
 	    t = getdata(REditors[i]);
 	    p = getdata(t);
 	    if (!strcmp (name, p->filename)) {
@@ -337,16 +388,17 @@ static void editoropen(char *default_name)
 		break;
 	    }
 	}
-	Rgui_Edit(name, name, 0);
-    }
+	reEnc2(name, title, MAX_PATH+1, CE_UTF8, CE_NATIVE, 3);
+	Rgui_Edit(name, CE_UTF8, title, 0);
+    } else show(RConsole);
 }
 
 void menueditoropen(control m)
 {
     editor c = getdata(m);
     char *default_name = "";
-    /* It really is not clear what is meant here: seems to assume an 
-       editor window but is called from elsewhere, hopefully with NULL 
+    /* It really is not clear what is meant here: seems to assume an
+       editor window but is called from elsewhere, hopefully with NULL
        (since 2.1.1 patched). */
     if (c) {
 	textbox t = getdata(c);
@@ -380,7 +432,8 @@ static void editorcopy(control m)
 static void editorpaste(control m)
 {
     textbox t = getdata(m);
-    /* check whether the widget text limit needs to be increased before doing the paste */
+    /* check whether the widget text limit needs to be increased
+     * before doing the paste */
     int pastelen = getpastelength();
     checklimittext(t, pastelen + 1);
     pastetext(t);
@@ -416,8 +469,10 @@ static void editorreplace(control m)
 static void editorrunline(textbox t)
 {
     int length = getlinelength(t); /* return character num */
-    char *line = malloc(length * sizeof(WCHAR) + 2); /* Extra space for null and word length in getcurrentline */
-    memset(line, 0, length * sizeof(WCHAR) + 2);
+    /* Extra space for null and word length in getcurrentline */
+    size_t alength = length * MB_CUR_MAX + 1 + sizeof(WORD);
+    char *line = malloc(alength);
+    memset(line, 0, alength);
     getcurrentline(t, line, length);
     consolecmd(RConsole, line);
     free(line);
@@ -444,20 +499,20 @@ static Rboolean busy_running = FALSE;
 static void editorrun(textbox t)
 {
     if (!busy_running) {
-        long start=0, end=0;
-    	if (CharacterMode != RGui) {
-    	    R_ShowMessage(G_("No RGui console to paste to"));
-    	    return;
-    	}
-    	busy_running = TRUE;
-    	textselectionex(t, &start, &end);
-    	if (start >= end)
+	long start=0, end=0;
+	if (CharacterMode != RGui) {
+	    R_ShowMessage(G_("No RGui console to paste to"));
+	    return;
+	}
+	busy_running = TRUE;
+	textselectionex(t, &start, &end);
+	if (start >= end)
 	    editorrunline(t);
-    	else {
+	else {
 	    editorrunselection(t, start, end);
 	    selecttextex(t, end, end); /* move insertion point to end of selection after running */
-    	}
-    	busy_running = FALSE;
+	}
+	busy_running = FALSE;
     }
 }
 
@@ -486,12 +541,12 @@ static void editormenuact(control m)
     EditorData p = getdata(t);
     textselectionex(t, &start, &end);
     if (start < end) {
-        enable(p->mcut);
-        enable(p->mcopy);
-        enable(p->mdelete);
-        enable(p->mpopcut);
-        enable(p->mpopcopy);
-        enable(p->mpopdelete);
+	enable(p->mcut);
+	enable(p->mcopy);
+	enable(p->mdelete);
+	enable(p->mpopcut);
+	enable(p->mpopcopy);
+	enable(p->mpopdelete);
     }
     else {
 	disable(p->mcut);
@@ -531,8 +586,8 @@ static void editorcontrolkeydown(textbox t, int key)
 static void editorasciikeydown(textbox t, int key)
 {
     /* check whether the text limit is about to be exceeded and
-     increase it if necessary.  Ugh - this callback is called after
-     the text is inserted, so we make space for two characters */
+       increase it if necessary.  Ugh - this callback is called after
+       the text is inserted, so we make space for two characters */
     checklimittext(t, 2);
 }
 
@@ -544,22 +599,22 @@ static void editorfocus(editor c)
     show(t);
 }
 
-static void editorhelp()
+static void editorhelp(void)
 {
     char s[4096];
 
-    strcpy(s,G_("R EDITOR\n"));
-    strcat(s,"\n");
-    strcat(s,G_("A standard text editor for editing and running R code.\n"));
-    strcat(s,"\n");
-    strcat(s,G_("RUNNING COMMANDS\n"));
-    strcat(s,G_("To run a line or section of R code, select the code and either\n"));
-    strcat(s,G_("     Press Ctrl-R\n"));
-    strcat(s,G_("     Select \"Run line or selection\" from the \"Edit\" menu\n"));
-    strcat(s,G_("     Press the \"Run line or selection\" icon on the toolbar\n"));
-    strcat(s,G_("This will copy the selected commands to the console and evaluate them.\n"));
-    strcat(s,G_("If there is no selection, this will just run the current line and advance\n"));
-    strcat(s,G_("the cursor by one line.\n"));
+    strcpy(s, G_("R EDITOR\n"));
+    strcat(s, "\n");
+    strcat(s, G_("A standard text editor for editing and running R code.\n"));
+    strcat(s, "\n");
+    strcat(s, G_("RUNNING COMMANDS\n"));
+    strcat(s, G_("To run a line or section of R code, select the code and either\n"));
+    strcat(s, G_("     Press Ctrl-R\n"));
+    strcat(s, G_("     Select \"Run line or selection\" from the \"Edit\" menu\n"));
+    strcat(s, G_("     Press the \"Run line or selection\" icon on the toolbar\n"));
+    strcat(s, G_("This will copy the selected commands to the console and evaluate them.\n"));
+    strcat(s, G_("If there is no selection, this will just run the current line and advance\n"));
+    strcat(s, G_("the cursor by one line.\n"));
 
     askok(s);
 }
@@ -584,7 +639,7 @@ static MenuItem EditorPopup[] = {                /* Numbers used below */
     LASTMENUITEM
 };
 
-static editor neweditor()
+static editor neweditor(void)
 {
     int x, y, w, h, w0, h0;
     editor c;
@@ -627,8 +682,10 @@ static editor neweditor()
     setdata(t, p);
 
     gsetcursor(c, ArrowCursor);
-    setforeground(c, consolefg);
-    setbackground(c, consolebg);
+    setforeground(c, guiColors[editorfg]);
+    setbackground(c, guiColors[editorbg]);
+    setbackground(t, guiColors[editorbg]);
+    
 #ifdef USE_MDI
     if (ismdi() && (RguiMDI & RW_TOOLBAR)) {
 	int btsize = 24;
@@ -683,14 +740,11 @@ static editor neweditor()
     MCHECK(m = newmenuitem(G_("Save as..."), 0, menueditorsaveas));
     setdata(m, c);
     MCHECK(m = newmenuitem("-", 0, NULL));
-    MCHECK(m = newmenuitem(G_("Print..."), 0, editorprint));
+    MCHECK(m = newmenuitem(G_("Print..."), 'P', editorprint));
     setdata(m, t);
     MCHECK(m = newmenuitem("-", 0, NULL));
     MCHECK(m = newmenuitem(G_("Close script"), 0, menueditorclose));
     setdata(m, c);
-    /* MCHECK(m = newmenuitem("-", 0, NULL));
-    MCHECK(m = newmenuitem(G_("Exit"), 0, closeconsole));
-    setdata(m, c); */
     MCHECK(newmenu(G_("Edit")));
     MCHECK(m = newmenuitem(G_("Undo"), 'Z', editorundo));
     setdata(m, t);
@@ -734,6 +788,7 @@ static editor neweditor()
     RguiCommonHelp(m, p->hmenu);
 
     settextfont(t, editorfn);
+    setforeground(t, guiColors[editorfg]);    
     setresize(c, editorresize);
     setclose(c, editorclose);
     setdel(c, editordel);
@@ -756,7 +811,7 @@ void editorsetfont(font f)
     textbox t;
     for (i = 0; i < neditors; i++) {
 	t = getdata(REditors[i]);
- 	ismod = ggetmodified(t);
+	ismod = ggetmodified(t);
 	/* Don't change the modification flag when changing font  */
 	settextfont(t, f);
 	gsetmodified(t, ismod);
@@ -768,7 +823,7 @@ static void eventloop(editor c)
 {
     while (fix_editor_up) {
 	/* avoid consuming 100% CPU time here */
-	Sleep(10);
+	R_WaitEvent();
 	R_ProcessEvents();
     }
 }
@@ -779,7 +834,8 @@ static void eventloop(editor c)
 
 #include <unistd.h>
 
-int Rgui_Edit(char *filename, char *title, int stealconsole)
+int Rgui_Edit(const char *filename, int enc, const char *title,
+	      int modal)
 {
     editor c;
     EditorData p;
@@ -794,20 +850,19 @@ int Rgui_Edit(char *filename, char *title, int stealconsole)
 	return 1;
     }
     if (strlen(filename) > 0) {
-	if (!access(filename, R_OK))
-	    editor_load_file(c, filename);
+	editor_load_file(c, filename, enc);
 	editor_set_title(c, title);
     }
     else {
 	editor_set_title(c, G_("Untitled"));
     }
     show(c);
-    
+
     p = getdata(getdata(c));
-    p->stealconsole = stealconsole;
-    if (stealconsole) {
-    	fix_editor_up = TRUE;
-    	eventloop(c);
+    p->stealconsole = modal;
+    if (modal) {
+	fix_editor_up = TRUE;
+	eventloop(c);
     }
     return 0;
 }
