@@ -1,4 +1,4 @@
-/*  nnet/src/nnet.c by W. N. Venables and B. D. Ripley  Copyright (C) 1992-2002
+/*  nnet/src/nnet.c by W. N. Venables and B. D. Ripley  Copyright (C) 1992-2022
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@
  *
  */
 
+#include <stdlib.h> // currently in R.h
 #include <R.h>
 #include <R_ext/Applic.h>
 #include <math.h>
@@ -69,9 +70,9 @@ static Sdata *Weights;
 static Sdata *toutputs;
 
 void
-VR_set_net(Sint *n, Sint *nconn, Sint *conn,
-	   double *decay, Sint *nsunits, Sint *entropy,
-	   Sint *softmax, Sint *censored)
+VR_set_net(int *n, int *nconn, int *conn,
+	   double *decay, int *nsunits, int *entropy,
+	   int *softmax, int *censored)
 {
     int   i;
 
@@ -79,11 +80,11 @@ VR_set_net(Sint *n, Sint *nconn, Sint *conn,
     for (i = 0; i <= Nunits; i++)
 	Nconn[i] = nconn[i];
     Nweights = Nconn[Nunits];
-    Conn = Calloc(Nweights, int);
-    wts = Calloc(Nweights, double);
-    Slopes = Calloc(Nweights, double);
-    Probs = Calloc(Nweights, double);
-    Decay = Calloc(Nweights, double);
+    Conn = R_Calloc(Nweights, int);
+    wts = R_Calloc(Nweights, double);
+    Slopes = R_Calloc(Nweights, double);
+    Probs = R_Calloc(Nweights, double);
+    Decay = R_Calloc(Nweights, double);
     for (i = 0; i < Nweights; i++)
 	Conn[i] = conn[i];
     Epoch = 0;
@@ -98,22 +99,22 @@ VR_set_net(Sint *n, Sint *nconn, Sint *conn,
 }
 
 void 
-VR_unset_net()
+VR_unset_net(void)
 {
-    Free(Conn);
-    Free(wts);
-    Free(Slopes);
-    Free(Probs);
-    Free(Decay);
-    Free(Nconn);
-    Free(Outputs);
-    Free(ErrorSums);
-    Free(Errors);
-    Free(toutputs);
+    R_Free(Conn);
+    R_Free(wts);
+    R_Free(Slopes);
+    R_Free(Probs);
+    R_Free(Decay);
+    R_Free(Nconn);
+    R_Free(Outputs);
+    R_Free(ErrorSums);
+    R_Free(Errors);
+    R_Free(toutputs);
 }
 
 void
-VR_nntest(Sint *ntest, Sdata *test, Sdata *result, double *inwts)
+VR_nntest(int *ntest, Sdata *test, Sdata *result, double *inwts)
 {
     int   i, j;
 
@@ -141,11 +142,11 @@ static void
 Build_Net(int ninputs, int nhidden, int noutputs)
 {
     Nunits = 1 + ninputs + nhidden + noutputs;
-    Nconn = Calloc(Nunits + 1, int);
-    Outputs = Calloc(Nunits, double);
-    ErrorSums = Calloc(Nunits, double);
-    Errors = Calloc(Nunits, double);
-    toutputs = Calloc(Nunits, Sdata);
+    Nconn = R_Calloc(Nunits + 1, int);
+    Outputs = R_Calloc(Nunits, double);
+    ErrorSums = R_Calloc(Nunits, double);
+    Errors = R_Calloc(Nunits, double);
+    toutputs = R_Calloc(Nunits, Sdata);
     Ninputs = ninputs;
     FirstHidden = 1 + ninputs;
     FirstOutput = 1 + ninputs + nhidden;
@@ -370,14 +371,14 @@ vect(int n)
 {
     double *v;
 
-    v = Calloc(n, double);
+    v = R_Calloc(n, double);
     return v;
 }
 
 static void 
 free_vect(double *v)
 {
-    Free(v);
+    R_Free(v);
 }
 
 static double **
@@ -386,10 +387,10 @@ matrix(int nrh, int nch)
     int   i;
     double **m;
 
-    m = Calloc((nrh + 1), double *);
+    m = R_Calloc((nrh + 1), double *);
 
     for (i = 0; i <= nrh; i++) {
-	m[i] = Calloc((nch + 1), double);
+	m[i] = R_Calloc((nch + 1), double);
     }
     return m;
 }
@@ -400,8 +401,8 @@ free_matrix(double **m, int nrh, int nch)
     int   i;
 
     for (i = nrh; i >= 0; i--)
-	Free(m[i]);
-    Free(m);
+	R_Free(m[i]);
+    R_Free(m);
 }
 
 static double **
@@ -410,10 +411,10 @@ Lmatrix(int n)
     int   i;
     double **m;
 
-    m = Calloc(n, double *);
+    m = R_Calloc(n, double *);
 
     for (i = 0; i < n; i++) {
-	m[i] = Calloc((i + 1), double);
+	m[i] = R_Calloc((i + 1), double);
     }
     return m;
 }
@@ -424,17 +425,17 @@ free_Lmatrix(double **m, int n)
     int   i;
 
     for (i = n - 1; i >= 0; i--)
-	Free(m[i]);
-    Free(m);
+	R_Free(m[i]);
+    R_Free(m);
 }
 
 
 #define REPORT		10
 
 void
-VR_dovm(Sint *ntr, Sdata *train, Sdata *weights,
-	Sint *Nw, double *wts, double *Fmin,
-	Sint *maxit, Sint *trace, Sint *mask,
+VR_dovm(int *ntr, Sdata *train, Sdata *weights,
+	int *Nw, double *wts, double *Fmin,
+	int *maxit, int *trace, int *mask,
 	double *abstol, double *reltol, int *ifail)
 {
     int fncount, grcount;
@@ -594,7 +595,7 @@ pHessian(Sdata *input, Sdata *goal, Sdata wx, int nr)
 #define min9(a,b) a<b?a:b
 
 void
-VR_nnHessian(Sint *ntr, Sdata *train, Sdata *weights,
+VR_nnHessian(int *ntr, Sdata *train, Sdata *weights,
 	     double *inwts, Sdata *Hess)
 {
     int   i, j;
@@ -651,7 +652,7 @@ Zcompar(const Sdata *a, const Sdata *b)
 /* Z is transposed, so (p+q) x n */
 
 void
-VR_summ2(Sint *n0, Sint *p0, Sint *q0, Sdata *Z, Sint *na)
+VR_summ2(int *n0, int *p0, int *q0, Sdata *Z, int *na)
 {
     int   n = *n0, m;
     int   i, j, k, l;
@@ -698,7 +699,5 @@ void R_init_nnet(DllInfo *dll)
 {
     R_registerRoutines(dll, CEntries, NULL, NULL, NULL);
     R_useDynamicSymbols(dll, FALSE);
-#if defined(R_VERSION) && R_VERSION >= R_Version(2, 16, 0)
     R_forceSymbols(dll, TRUE);
-#endif
 }
