@@ -5,7 +5,7 @@
 if(interactive()) {
     options(error = recover, warn = 1)
 } else if(FALSE) { ## MM / developer  testing *manually* :
-    options(error = recover, Matrix.verbose = 2,   warn = 1)
+    options(error = recover, Matrix.verbose = 2, warn = 1)
 } else {
     options(                 Matrix.verbose = 2, warn = 1)
 }
@@ -18,7 +18,11 @@ stopifnot(suppressPackageStartupMessages(require(Matrix)))
 source(system.file("test-tools.R", package = "Matrix"), keep.source = FALSE)
 ##-> identical3() etc
 cat("doExtras:",doExtras,"\n")
-
+if(exists("Sys.setLanguage", mode="function"))
+    Sys.setLanguage("en")
+englishMsgs <- (lang <- Sys.getenv("LANGUAGE")) %in% c("en", "C")
+cat(sprintf("LANGUAGE env.: '%s'; englishMsgs: %s\n",
+            lang, englishMsgs))
 
 ### Dense Matrices
 
@@ -190,7 +194,7 @@ chkAssign <- function(ms, mm = as(ms, "matrix"),
     ms[i,j] <- v
     ## useful to see (ii,ij), but confusing R/ESS when additionally debugging:
     ## if(!showOnly && interactive()) { op <- options(error = recover); on.exit(options(op)) }
-    assert.EQ.mat(ms, mm, show=showOnly)
+    assert.EQ.mat(ms, mm, showOnly=showOnly)
     ## vector indexing m[cbind(i,j)] == m[i + N(j-1)] ,  N = nrow(.)
     ii <- seq_len(min(length(i), length(j)))
     i <- i[ii]
@@ -214,7 +218,7 @@ chkAssign <- function(ms, mm = as(ms, "matrix"),
 	ms2[neg.ii] <- v[ord.i]
 	stopifnot(identical(ms2, ms.))
     }
-    assert.EQ.mat(ms., mm., show=showOnly)
+    assert.EQ.mat(ms., mm., showOnly=showOnly)
 } ##{chkAssign}
 
 ## Get duplicated index {because these are "hard" (and rare)
@@ -236,21 +240,25 @@ for(kind in c("n", "l", "d")) {
     kClass <-paste0(kind, "Matrix"  )
     Ckind <- paste0(kind, "gCMatrix")
     Tkind <- paste0(kind, "gTMatrix")
-    str(mC <- as(m, Ckind))
-    str(mT <- as(as(as(m, kClass), "TsparseMatrix"), Tkind))
+    str(mC <- as(as(as(m, kClass), "CsparseMatrix"), "generalMatrix")) # was as(m, Ckind) deprecated
+    #was mT <- as(as(as(m, kClass), "TsparseMatrix"), Tkind))
+    str(mT <- as(as(as(m, kClass), "generalMatrix"), "TsparseMatrix"))
     mm <- as(mC, "matrix") # also logical or double
     IDENT <- if(kind == "n") function(x,y) Q.eq2(x,y, tol=0) else identical
-    stopifnot(identical(mT, as(as(mC, "TsparseMatrix"), Tkind)),
-              identical(mC, as(mT, Ckind)),
-              Qidentical(mC[0,0], new(Ckind)),
-              Qidentical(mT[0,0], new(Tkind)),
-              identical(unname(mT[0,]), new(Tkind, Dim = c(0L,ncol(m)))),
-              identical(unname(mT[,0]), new(Tkind, Dim = c(nrow(m),0L))),
-              IDENT(mC[0,], as(mT[FALSE,], Ckind)),
-              IDENT(mC[,0], as(mT[,FALSE], Ckind)),
-              sapply(pmin(min(dim(mC)), c(0:2, 5:10)),
-                     function(k) {i <- seq_len(k); all(mC[i,i] == mT[i,i])}),
-              TRUE)
+    stopifnot(exprs = {
+        identical(mT, as(mC, "TsparseMatrix"))
+        identical(mC, as(mT, "CsparseMatrix")) # was Ckind; now deprecated
+        Qidentical(mC[0,0], new(Ckind)) # M..3 Csp..4
+        Qidentical(mT[0,0], new(Tkind)) #  "
+        identical(unname(mT[0,]), new(Tkind, Dim = c(0L,ncol(m))))# M.3 T.4 C.4
+        identical(unname(mT[,0]), new(Tkind, Dim = c(nrow(m),0L)))# M.3 C.4
+        is.null(cat("IDENT():\n"))
+        IDENT(mC[0,], as(mT[FALSE,], "CsparseMatrix")) # M.3 C.4 M.3 + Tsp..4 Csp..4 | as(., Ckind) deprecated
+        IDENT(mC[,0], as(mT[,FALSE], "CsparseMatrix")) # M.3 C.4 M.3 C.4             | as(., Ckind) deprecated
+        is.null(cat("sapply(..):\n"))
+        sapply(pmin(min(dim(mC)), c(0:2, 5:10)),
+               function(k) {i <- seq_len(k); all(mC[i,i] == mT[i,i])})
+    })
     cat("ok\n")
     show(mC[,1])
     show(mC[1:2,])
@@ -348,11 +356,14 @@ if(doExtras) {### {was ./AAA_index.R, MM-only}
     print(load(system.file("external", "symA.rda", package="Matrix"))) # "As"
     stopifnotValid(As, "dsCMatrix"); stopifnot(identical(As@factors, list()))
     R. <- drop0(chol(As))
-    stopifnot(1:32 == sort(diag(R.)), ## !
-              R.@x == as.integer(R.@x),## so it is an integer-valued chol-decomp !
-              ## shows that (1) As is *not* singular  (2) the matrix is not random
-              all.equal(crossprod(R.), As, tolerance =1e-15))
-    print(summary(evA <- eigen(As, only.values=TRUE)$val))
+    stopifnot(exprs = {
+        1:32 == sort(diag(R.))  ## !
+        R.@x > 0
+        R.@x == as.integer(R.@x)## so it is an integer-valued chol-decomp !
+        ## shows that (1) As is *not* singular  (2) the matrix is not random
+        all.equal(crossprod(R.), As, tolerance=1e-15)
+    })
+    print(summary(evA <- eigen(As, only.values=TRUE)$values))
     print(tail(evA)) ## largest three ~= 10^7,  smallest two *negative*
     print(rcond(As)) # 1.722 e-21 == very bad !
     ##-> this *is* a border line case, i.e. very close to singular !
@@ -392,6 +403,7 @@ i <- c(7:5, 2:4);assert.EQ.mat(T[i,i], ss[i,i])
 ## NA in indices  -- check that we get a helpful error message:
 i[2] <- NA
 er <- tryCatch(T[i,i], error = function(e)e)
+if(englishMsgs)
 stopifnot(as.logical(grep("indices.*sparse Matrices", er$message)))
 
 N <- nrow(T)
@@ -432,7 +444,7 @@ a. <- as(A., "matrix")
 ## More testing {this was not working for a long time..}
 set.seed(1)
 for(n in 1:(if(doExtras) 100 else 6)) {
-    i <- sample(1:nrow(A), 3+2*rpois(1, lam=3), replace=TRUE)
+    i <- sample(1:nrow(A), 3+2*rpois(1, lambda=3), replace=TRUE)
     Aii  <- A[i,i]
     A.ii <- A.[i,i]
     stopifnot(class(Aii) == class(A),
@@ -476,11 +488,11 @@ x.x <- crossprod(mC)
 stopifnot(class(x.x) == "dsCMatrix",
           class(x.x. <- round(x.x / 10000)) == "dsCMatrix",
           identical(x.x[cbind(2:6, 2:6)],
-                    diag(x.x [2:6, 2:6])))
+                    diag(x.x[2:6, 2:6], names=FALSE)))
 head(x.x.) # Note the *non*-structural 0's printed as "0"
 tail(x.x., -3) # all but the first three lines
 
-lx.x <- as(x.x, "lsCMatrix") # FALSE only for "structural" 0
+lx.x <- as(as(x.x, "lMatrix"), "symmetricMatrix") # FALSE only for "structural" 0
 (l10 <- lx.x[1:10, 1:10])# "lsC"
 (l3 <-  lx.x[1:3, ])
 m.x <- as.mat(x.x) # as.mat() *drops* (NULL,NULL) dimnames
@@ -599,7 +611,6 @@ stopifnot(identical(m2[1:3,], as(m1[1:3,], "CsparseMatrix")),
 ## 0-dimensional diagonal - subsetting ----------------------------
 ## before that diagU2N() etc for 0-dim. dtC*:
 m0. <- m00 <- matrix(numeric(),0,0)
-dimnames(m0.) <- list(NULL, NULL)
  tC0.<- new("dtCMatrix")
  tC0 <- new("dtCMatrix", diag="U")
 (gC0 <- new("dgCMatrix")) # 0 x 0
@@ -647,6 +658,7 @@ EE <- lapply(expr[-1], function(e)
 exR <- lapply(EE, `[[`, "r")
 stopifnot(exprs = {
     vapply(exR, inherits, logical(1), what = "error")
+    !englishMsgs ||
     unique( vapply(exR, `[[`, "<msg>", "message")
            ) == "logical subscript too long (1, should be 0)"
 })
@@ -697,7 +709,8 @@ asLogical <- function(x) {
 }
 .iniDiag.example <- expression({
     D <- Diagonal(6)
-    M <- as(D,"dgeMatrix")
+    M <- as(D,"generalMatrix") # was "dge", now "dgC"
+    d <- as(D,"unpackedMatrix")  # "dtr" (unitri)
     m <- as(D,"matrix")
     s <- as(D,"TsparseMatrix"); N <- as(s,"nMatrix")
     S <- as(s,"CsparseMatrix"); C <- as(S,"nMatrix")
@@ -710,7 +723,7 @@ L <- c(TRUE, rep(FALSE,8)) ; z <- c(50,99)
 ## vector subassignment, both with integer & logical
 ## these now work correctly {though not very efficiently; hence warnings}
 m[i] <- v # the role model: only first column is affected
-M[i] <- v; assert.EQ.mat(M,m) # dge
+M[i] <- v; assert.EQ.mat(M,m) # dgC
 D[i] <- v; assert.EQ.mat(D,m) # ddi -> dtC (new! 2019-07; was dgT)
 s[i] <- v; assert.EQ.mat(s,m) # dtT -> dgT
 S[i] <- v; assert.EQ.mat(S,m); S # dtC -> dtT -> dgT -> dgC
@@ -786,8 +799,9 @@ assert.EQ.mat(N,m.L); N ; assertWarning(
 C[i] <- v, verbose=TRUE)
 assert.EQ.mat(C,m.L); C #
 
-options(warn = 2) #---------------------# NO WARNINGS from here -----------------
-					# =====================
+options(warn = 2) #----------------------# NO WARNINGS from here -----------------
+					## =====================
+
 ## 2) negative [i,j] indices
 mc <- mC[1:5, 1:7]
 mt <- mT[1:5, 1:7]
@@ -880,7 +894,7 @@ mc
 
 S <- as(Diagonal(5),"TsparseMatrix")
 H <- Hilbert(9)
-Hc <- as(round(H, 3), "dsCMatrix")# a sparse matrix with no 0 ...
+Hc <- as(round(H, 3), "CsparseMatrix")# a sparse matrix with no 0 ...
 (trH <- tril(Hc[1:5, 1:5]))
 stopifnot(is(trH, "triangularMatrix"), trH@uplo == "L",
           is(S, "triangularMatrix"))
@@ -898,7 +912,7 @@ trH[ lower.tri(trH) ] <- 0   # ditto, because of callNextMethod()
 
 m <- Matrix(0+1:28, nrow = 4)
 m[-3,c(2,4:5,7)] <- m[ 3, 1:4] <- m[1:3, 6] <- 0
-mT <- as(m, "dgTMatrix")
+mT <- as(m, "TsparseMatrix")
 stopifnot(identical(mT[lower.tri(mT)],
                     m [lower.tri(m) ]))
 lM <- upper.tri(mT, diag=TRUE)
@@ -1090,6 +1104,9 @@ stopifnot(identical(Diagonal(x = 1+ 1:n), dLrg),
 cc <- capture.output(show(dLrg))# show(<diag>) used to error for large n
 showProc.time()
 
+## FIXME: "dspMatrix" (symmetric *packed*) not going via "matrix"
+
+
 ## Large Matrix indexing / subassignment
 ## ------------------------------------- (from ex. by Imran Rashid)
 n <- 7000000
@@ -1120,7 +1137,7 @@ fx <- sparseMatrix(i = sample(n, size=nnz, replace=TRUE),
                    j = sample(m, size=nnz, replace=TRUE),
                    x = round(10*rnorm(nnz)))
 class(fx)## dgCMatrix
-fx[,6000] <- (tC <- rep(thisCol, length=nrow(fx)))# fine
+fx[,6000] <- (tC <- rep(thisCol, length.out=nrow(fx)))# fine
 thCol <- fx[,2000]
 fx[,5762] <- thCol# fine
 stopifnot(is(f, "ngCMatrix"), is(fx, "dgCMatrix"),
@@ -1145,10 +1162,10 @@ showProc.time()
 
 ## Bugs found by Peter Ralph
 n <- 17
-x <- Matrix(0, n,n)
+x <- Matrix(0, n,n) # "ddiMatrix" now
 ## x must have at least three nonzero entries
-x[1,1] <- x[2,1:2] <- 1.
-x0 <- x <- as(x,"dgTMatrix")  # if x is dgCMatrix, no error
+x[1,1] <- x[2,1:2] <- 1.; class(x) # "dtC"
+x0 <- x <- as(x,"generalMatrix")  # if x is dgCMatrix, no error
 ##
 z <- matrix(x) # <== not the "Matrix way":  a (n, 1) matrix
 z[1] <- 0
@@ -1157,7 +1174,7 @@ x[1:n, 1:n] <- as(z, "sparseVector")
 ## gave Error: ... invalid subscript type 'S4'
 x2 <- x
 
-dim(zC <- as(z, "dgCMatrix"))
+dim(zC <- as(z, "CsparseMatrix"))
 x <- x0
 x[] <- zC # did fail, then gave warning.
 x1 <- x
@@ -1175,9 +1192,45 @@ x1 <- x0; x1[cbind(i, i+10)] <- i^2
 x2 <- x0; x2[cbind(i, i+10)] <- .asmatrix(i^2)
 ## failed: nargs() = 4 ... please report
 
-stopifnot(isValid(x1, "dgTMatrix"), identical(x1, x2))
+class(x1) # was "dgT", now "dgC"
+stopifnot(isValid(x1, class(x1)), identical(x1, x2))
+showProc.time()
 
 
+## check valid indexing (using *random* indices, often duplicated):
+chk_dsp_ind <- function(sv, n=512, negI = FALSE, verbose=FALSE) {
+    stopifnot(inherits(sv, "dsparseVector"), n >= 1)
+    d <- length(sv)
+    ## lambda=2 ==> aiming for short 'i' {easier to work with}
+    P <- rpois(n, lambda = if(negI) 5 else 2)
+    for(i in seq_len(n)) {
+        I <-
+            if(negI) { # negative indices: 2 are, 4 neither ... always "valid" !!
+                k <- max(4L, d - max(1L, P[i]))
+                if(verbose) cat(sprintf("size=k = %2d: ", k))
+                - sort(sample.int(d, size=k))# replace=FALSE
+            }
+            else
+                sample.int(d, size=1L+P[i], replace=TRUE)
+        ##
+        validObject(ss <- sv[I]) # Error if not true
+    }
+    invisible()
+}
+s <- as(c(3,5,6), "sparseVector")
+set.seed(11); chk_dsp_ind(s)
+set.seed(3)
+(s2 <- as(rsparsematrix(ncol=1, nrow=37, density=1/4),"sparseVector"))
+(s3 <- as(rsparsematrix(ncol=1, nrow=64, density=1/4),"sparseVector"))
+set.seed(1)
+chk_dsp_ind(s2)
+chk_dsp_ind(s3)
+##
+set.seed(47)
+## system.time(e.N2 <- chk_dsp_ind(s2, negI=TRUE, verbose=TRUE))
+chk_dsp_ind(s2, negI=TRUE)
+chk_dsp_ind(s3, negI=TRUE)
+
 iv <- c(rep(0,100), 3, 0,0,7,0,0,0)
 sv0  <- sv  <- as(iv, "sparseVector")
 sv.0 <- sv. <- as(as.integer(iv), "sparseVector")
@@ -1269,3 +1322,32 @@ Cx[2,] <- 0
 Rx[2,] <- 0
 stopifnot(all(Cx == X),
           all(Rx == X))
+
+## [matrix-Bugs][#6745] show(<large sparseVector>)
+## NB: is from a bug in head(*); *only* applies to  *empty* sparseV: length(x@i) == 0
+op <- options(max.print=999)
+( s0 <- sparseVector(i=integer(), length=2^33)) # show -> head() failed in Matrix <= 1.3-*
+(xs0 <- sparseVector(i=integer(), length=2^33, x = numeric()))# ditto
+options(op); tail(s0) ; tail(xs0) # (always worked)
+## *related* bug in `[` --> needed to fix intIv() for such large sparseVectors
+stopifnot(exprs = {
+    identical(s0[length(s0) - 3:0], # gave Error in if (any(i < 0L)) { : missing value ....
+              new("nsparseVector", i=integer(), length=4L))
+    identical(xs0[length(s0) - 3:0], # gave Error ..
+              new("dsparseVector", i=integer(), length=4L))
+})
+
+## Yielded an invalid object in Matrix <= 1.4-1, instead of throwing error
+llc04 <- new("dgCMatrix", Dim = c(4L, 0L))
+c40 <- new("dgCMatrix", Dim = c(0L, 4L), p = integer(5L))
+assertError(c04[1L, ] <- 1)
+assertError(c40[, 1L] <- 1)
+
+## Indexing with nMatrix rather than lMatrix
+set.seed(3601)
+gC <- rsparsematrix(6, 6, 0.6)
+gC@x[sample.int(length(gC@x), 6L)] <- NA
+ni <- is.na(gC)
+li <- as(ni, "lMatrix")
+stopifnot(identical(gC[ ni], gC[ li]),
+          identical(gC[!ni], gC[!li]))
